from __future__ import annotations

import datetime
from datetime import timezone

import pytest

from hcloud.volumes import Volume


@pytest.mark.parametrize(
    "value",
    [
        (Volume(id=1),),
    ],
)
def test_eq(value):
    assert value.__eq__(value)


class TestVolume:
    def test_created_is_datetime(self):
        volume = Volume(id=1, created="2016-01-30T23:50+00:00")
        assert volume.created == datetime.datetime(
            2016, 1, 30, 23, 50, tzinfo=timezone.utc
        )



from __future__ import annotations

import pytest

from hcloud.storage_boxes import StorageBox


@pytest.mark.parametrize(
    "value",
    [
        (StorageBox(id=1),),
    ],
)
def test_eq(value):
    assert value.__eq__(value)



from __future__ import annotations

from unittest import mock

import pytest

from hcloud import Client
from hcloud.load_balancer_types import LoadBalancerTypesClient


class TestLoadBalancerTypesClient:
    @pytest.fixture()
    def load_balancer_types_client(self, client: Client):
        return LoadBalancerTypesClient(client)

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        load_balancer_types_client: LoadBalancerTypesClient,
        load_balancer_type_response,
    ):
        request_mock.return_value = load_balancer_type_response

        load_balancer_type = load_balancer_types_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/load_balancer_types/1",
        )
        assert load_balancer_type._client is load_balancer_types_client
        assert load_balancer_type.id == 1
        assert load_balancer_type.name == "LB11"

    @pytest.mark.parametrize(
        "params", [{"name": "LB11", "page": 1, "per_page": 10}, {"name": ""}, {}]
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        load_balancer_types_client: LoadBalancerTypesClient,
        two_load_balancer_types_response,
        params,
    ):
        request_mock.return_value = two_load_balancer_types_response

        result = load_balancer_types_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/load_balancer_types",
            params=params,
        )

        load_balancer_types = result.load_balancer_types
        assert result.meta is not None

        assert len(load_balancer_types) == 2

        load_balancer_types1 = load_balancer_types[0]
        load_balancer_types2 = load_balancer_types[1]

        assert load_balancer_types1._client is load_balancer_types_client
        assert load_balancer_types1.id == 1
        assert load_balancer_types1.name == "LB11"

        assert load_balancer_types2._client is load_balancer_types_client
        assert load_balancer_types2.id == 2
        assert load_balancer_types2.name == "LB21"

    @pytest.mark.parametrize("params", [{"name": "LB21"}])
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        load_balancer_types_client: LoadBalancerTypesClient,
        two_load_balancer_types_response,
        params,
    ):
        request_mock.return_value = two_load_balancer_types_response

        load_balancer_types = load_balancer_types_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/load_balancer_types",
            params=params,
        )

        assert len(load_balancer_types) == 2

        load_balancer_types1 = load_balancer_types[0]
        load_balancer_types2 = load_balancer_types[1]

        assert load_balancer_types1._client is load_balancer_types_client
        assert load_balancer_types1.id == 1
        assert load_balancer_types1.name == "LB11"

        assert load_balancer_types2._client is load_balancer_types_client
        assert load_balancer_types2.id == 2
        assert load_balancer_types2.name == "LB21"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        load_balancer_types_client: LoadBalancerTypesClient,
        one_load_balancer_types_response,
    ):
        request_mock.return_value = one_load_balancer_types_response

        load_balancer_type = load_balancer_types_client.get_by_name("LB21")

        params = {"name": "LB21"}

        request_mock.assert_called_with(
            method="GET",
            url="/load_balancer_types",
            params=params,
        )

        assert load_balancer_type._client is load_balancer_types_client
        assert load_balancer_type.id == 2
        assert load_balancer_type.name == "LB21"



from __future__ import annotations

from unittest import mock

import pytest

from hcloud import Client
from hcloud.datacenters import BoundDatacenter, Datacenter
from hcloud.primary_ips import BoundPrimaryIP, PrimaryIP, PrimaryIPsClient

from ..conftest import BoundModelTestCase


class TestBoundPrimaryIP(BoundModelTestCase):
    methods = [
        BoundPrimaryIP.update,
        BoundPrimaryIP.delete,
        BoundPrimaryIP.change_dns_ptr,
        BoundPrimaryIP.change_protection,
        BoundPrimaryIP.assign,
        BoundPrimaryIP.unassign,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.primary_ips

    @pytest.fixture()
    def bound_model(self, resource_client: PrimaryIPsClient):
        return BoundPrimaryIP(resource_client, data=dict(id=14))

    def test_init(self, primary_ip_response):
        bound_primary_ip = BoundPrimaryIP(
            client=mock.MagicMock(), data=primary_ip_response["primary_ip"]
        )

        assert bound_primary_ip.id == 42
        assert bound_primary_ip.name == "my-resource"
        assert bound_primary_ip.ip == "131.232.99.1"
        assert bound_primary_ip.type == "ipv4"
        assert bound_primary_ip.protection == {"delete": False}
        assert bound_primary_ip.labels == {}
        assert bound_primary_ip.blocked is False

        assert bound_primary_ip.assignee_id == 17
        assert bound_primary_ip.assignee_type == "server"

        with pytest.deprecated_call():
            datacenter = bound_primary_ip.datacenter

        assert isinstance(datacenter, BoundDatacenter)
        assert datacenter.id == 42
        assert datacenter.name == "fsn1-dc8"
        assert datacenter.description == "Falkenstein DC Park 8"
        assert datacenter.location.country == "DE"
        assert datacenter.location.city == "Falkenstein"
        assert datacenter.location.latitude == 50.47612
        assert datacenter.location.longitude == 12.370071


class TestPrimaryIPsClient:
    @pytest.fixture()
    def primary_ips_client(self, client: Client):
        return PrimaryIPsClient(client)

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        primary_ip_response,
    ):
        request_mock.return_value = primary_ip_response

        bound_primary_ip = primary_ips_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/primary_ips/1",
        )
        assert bound_primary_ip._client is primary_ips_client
        assert bound_primary_ip.id == 42

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        one_primary_ips_response,
    ):
        request_mock.return_value = one_primary_ips_response

        bound_primary_ip = primary_ips_client.get_by_name("my-resource")

        request_mock.assert_called_with(
            method="GET",
            url="/primary_ips",
            params={"name": "my-resource"},
        )
        assert bound_primary_ip._client is primary_ips_client
        assert bound_primary_ip.id == 42
        assert bound_primary_ip.name == "my-resource"

    @pytest.mark.parametrize("params", [{"label_selector": "label1"}])
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        all_primary_ips_response,
        params,
    ):
        request_mock.return_value = all_primary_ips_response

        bound_primary_ips = primary_ips_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/primary_ips",
            params=params,
        )

        assert len(bound_primary_ips) == 1

        bound_primary_ip1 = bound_primary_ips[0]

        assert bound_primary_ip1._client is primary_ips_client
        assert bound_primary_ip1.id == 42
        assert bound_primary_ip1.name == "my-resource"

    def test_create_with_datacenter(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        primary_ip_response,
    ):
        request_mock.return_value = primary_ip_response

        with pytest.deprecated_call():
            response = primary_ips_client.create(
                type="ipv6",
                name="my-resource",
                datacenter=Datacenter(name="datacenter"),
            )

        request_mock.assert_called_with(
            method="POST",
            url="/primary_ips",
            json={
                "name": "my-resource",
                "type": "ipv6",
                "assignee_type": "server",
                "datacenter": "datacenter",
                "auto_delete": False,
            },
        )

        bound_primary_ip = response.primary_ip
        action = response.action

        assert bound_primary_ip._client is primary_ips_client
        assert bound_primary_ip.id == 42
        assert bound_primary_ip.name == "my-resource"
        assert action is None

    def test_create_with_assignee_id(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        primary_ip_create_response,
    ):
        request_mock.return_value = primary_ip_create_response

        response = primary_ips_client.create(
            type="ipv6",
            name="my-ip",
            assignee_id=17,
            assignee_type="server",
        )

        request_mock.assert_called_with(
            method="POST",
            url="/primary_ips",
            json={
                "name": "my-ip",
                "type": "ipv6",
                "assignee_id": 17,
                "assignee_type": "server",
                "auto_delete": False,
            },
        )
        bound_primary_ip = response.primary_ip
        action = response.action

        assert bound_primary_ip._client is primary_ips_client
        assert bound_primary_ip.id == 42
        assert bound_primary_ip.name == "my-ip"
        assert bound_primary_ip.assignee_id == 17
        assert action.id == 13

    @pytest.mark.parametrize(
        "primary_ip", [PrimaryIP(id=1), BoundPrimaryIP(mock.MagicMock(), dict(id=1))]
    )
    def test_update(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        primary_ip,
        response_update_primary_ip,
    ):
        request_mock.return_value = response_update_primary_ip

        primary_ip = primary_ips_client.update(
            primary_ip, auto_delete=True, name="my-resource"
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/primary_ips/1",
            json={"auto_delete": True, "name": "my-resource"},
        )

        assert primary_ip.id == 42
        assert primary_ip.auto_delete is True
        assert primary_ip.name == "my-resource"

    @pytest.mark.parametrize(
        "primary_ip", [PrimaryIP(id=1), BoundPrimaryIP(mock.MagicMock(), dict(id=1))]
    )
    def test_change_protection(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        primary_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        action = primary_ips_client.change_protection(primary_ip, True)

        request_mock.assert_called_with(
            method="POST",
            url="/primary_ips/1/actions/change_protection",
            json={"delete": True},
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "primary_ip", [PrimaryIP(id=1), BoundPrimaryIP(mock.MagicMock(), dict(id=1))]
    )
    def test_delete(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        primary_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        delete_success = primary_ips_client.delete(primary_ip)

        request_mock.assert_called_with(
            method="DELETE",
            url="/primary_ips/1",
        )

        assert delete_success is True

    @pytest.mark.parametrize(
        "assignee_id,assignee_type,primary_ip",
        [
            (1, "server", PrimaryIP(id=12)),
            (1, "server", BoundPrimaryIP(mock.MagicMock(), dict(id=12))),
        ],
    )
    def test_assign(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        assignee_id,
        assignee_type,
        primary_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        action = primary_ips_client.assign(primary_ip, assignee_id, assignee_type)

        request_mock.assert_called_with(
            method="POST",
            url="/primary_ips/12/actions/assign",
            json={"assignee_id": 1, "assignee_type": "server"},
        )
        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "primary_ip", [PrimaryIP(id=12), BoundPrimaryIP(mock.MagicMock(), dict(id=12))]
    )
    def test_unassign(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        primary_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        action = primary_ips_client.unassign(primary_ip)

        request_mock.assert_called_with(
            method="POST",
            url="/primary_ips/12/actions/unassign",
        )
        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "primary_ip", [PrimaryIP(id=12), BoundPrimaryIP(mock.MagicMock(), dict(id=12))]
    )
    def test_change_dns_ptr(
        self,
        request_mock: mock.MagicMock,
        primary_ips_client: PrimaryIPsClient,
        primary_ip,
        action_response,
    ):
        request_mock.return_value = action_response

        action = primary_ips_client.change_dns_ptr(
            primary_ip, "1.2.3.4", "server02.example.com"
        )

        request_mock.assert_called_with(
            method="POST",
            url="/primary_ips/12/actions/change_dns_ptr",
            json={"ip": "1.2.3.4", "dns_ptr": "server02.example.com"},
        )
        assert action.id == 1
        assert action.progress == 0



from __future__ import annotations

from unittest import mock

import pytest  # noqa: F401

from hcloud import Client
from hcloud.locations import LocationsClient


class TestLocationsClient:
    @pytest.fixture()
    def locations_client(self, client: Client):
        return LocationsClient(client)

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        locations_client: LocationsClient,
        location_response,
    ):
        request_mock.return_value = location_response

        location = locations_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/locations/1",
        )
        assert location._client is locations_client
        assert location.id == 1
        assert location.name == "fsn1"
        assert location.network_zone == "eu-central"

    @pytest.mark.parametrize(
        "params", [{"name": "fsn1", "page": 1, "per_page": 10}, {"name": ""}, {}]
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        locations_client: LocationsClient,
        two_locations_response,
        params,
    ):
        request_mock.return_value = two_locations_response

        result = locations_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/locations",
            params=params,
        )

        locations = result.locations
        assert result.meta is not None

        assert len(locations) == 2

        location1 = locations[0]
        location2 = locations[1]

        assert location1._client is locations_client
        assert location1.id == 1
        assert location1.name == "fsn1"
        assert location1.network_zone == "eu-central"

        assert location2._client is locations_client
        assert location2.id == 2
        assert location2.name == "nbg1"
        assert location2.network_zone == "eu-central"

    @pytest.mark.parametrize("params", [{"name": "fsn1"}, {}])
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        locations_client: LocationsClient,
        two_locations_response,
        params,
    ):
        request_mock.return_value = two_locations_response

        locations = locations_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/locations",
            params=params,
        )

        assert len(locations) == 2

        location1 = locations[0]
        location2 = locations[1]

        assert location1._client is locations_client
        assert location1.id == 1
        assert location1.name == "fsn1"
        assert location1.network_zone == "eu-central"

        assert location2._client is locations_client
        assert location2.id == 2
        assert location2.name == "nbg1"
        assert location2.network_zone == "eu-central"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        locations_client: LocationsClient,
        one_locations_response,
    ):
        request_mock.return_value = one_locations_response

        location = locations_client.get_by_name("fsn1")

        params = {"name": "fsn1"}

        request_mock.assert_called_with(
            method="GET",
            url="/locations",
            params=params,
        )

        assert location._client is locations_client
        assert location.id == 1
        assert location.name == "fsn1"
        assert location.network_zone == "eu-central"



from __future__ import annotations

import inspect
from unittest import mock

import pytest

from hcloud import Client
from hcloud.actions import (
    ActionFailedException,
    ActionsClient,
    ActionTimeoutException,
    BoundAction,
    ResourceActionsClient,
)
from hcloud.certificates import BoundCertificate, CertificatesClient
from hcloud.core import BoundModelBase, ResourceClientBase
from hcloud.firewalls import BoundFirewall, FirewallsClient
from hcloud.floating_ips import BoundFloatingIP, FloatingIPsClient
from hcloud.images import BoundImage, ImagesClient
from hcloud.load_balancers import BoundLoadBalancer, LoadBalancersClient
from hcloud.networks import BoundNetwork, NetworksClient
from hcloud.primary_ips import BoundPrimaryIP, PrimaryIPsClient
from hcloud.servers import BoundServer, ServersClient
from hcloud.storage_boxes import BoundStorageBox, StorageBoxesClient
from hcloud.volumes import BoundVolume, VolumesClient
from hcloud.zones import BoundZone, ZonesClient

from ..conftest import assert_bound_action1, assert_bound_action2

resources_with_actions: dict[str, tuple[ResourceClientBase, BoundModelBase]] = {
    "certificates": (CertificatesClient, BoundCertificate),
    "firewalls": (FirewallsClient, BoundFirewall),
    "floating_ips": (FloatingIPsClient, BoundFloatingIP),
    "images": (ImagesClient, BoundImage),
    "load_balancers": (LoadBalancersClient, BoundLoadBalancer),
    "networks": (NetworksClient, BoundNetwork),
    "primary_ips": (PrimaryIPsClient, BoundPrimaryIP),
    "servers": (ServersClient, BoundServer),
    "volumes": (VolumesClient, BoundVolume),
    "zones": (ZonesClient, BoundZone),
    "storage_boxes": (StorageBoxesClient, BoundStorageBox),
}


def test_resources_with_actions(client: Client):
    """
    Ensure that the list of resource clients above is up to date.
    """
    members = inspect.getmembers(
        client,
        predicate=lambda p: isinstance(p, ResourceClientBase) and hasattr(p, "actions"),
    )
    for name, member in members:
        assert name in resources_with_actions

        resource_client_class, _ = resources_with_actions[name]
        assert member.__class__ is resource_client_class

    assert len(members) == len(resources_with_actions)


class TestBoundAction:
    @pytest.fixture()
    def bound_running_action(self, client: Client, action1_running):
        return BoundAction(client=client.actions, data=action1_running)

    def test_wait_until_finished(
        self,
        request_mock: mock.MagicMock,
        bound_running_action,
        action1_running,
        action1_success,
    ):
        request_mock.side_effect = [
            {"action": action1_running},
            {"action": action1_success},
        ]

        bound_running_action.wait_until_finished()

        request_mock.assert_called_with(
            method="GET",
            url="/actions/1",
        )

        assert bound_running_action.status == "success"
        assert bound_running_action.id == 1

        assert request_mock.call_count == 2

    def test_wait_until_finished_with_error(
        self,
        request_mock: mock.MagicMock,
        bound_running_action,
        action1_running,
        action1_error,
    ):
        request_mock.side_effect = [
            {"action": action1_running},
            {"action": action1_error},
        ]

        with pytest.raises(ActionFailedException) as exc:
            bound_running_action.wait_until_finished()

        assert bound_running_action.status == "error"
        assert bound_running_action.id == 1
        assert exc.value.action.id == 1

        assert request_mock.call_count == 2

    def test_wait_until_finished_max_retries(
        self,
        request_mock: mock.MagicMock,
        bound_running_action,
        action1_running,
        action1_success,
    ):
        request_mock.side_effect = [
            {"action": action1_running},
            {"action": action1_running},
            {"action": action1_success},
        ]

        with pytest.raises(ActionTimeoutException) as exc:
            bound_running_action.wait_until_finished(max_retries=1)

        assert bound_running_action.status == "running"
        assert bound_running_action.id == 1
        assert exc.value.action.id == 1

        assert request_mock.call_count == 1


class TestResourceActionsClient:
    """
    /<resource>/actions
    /<resource>/actions/<id>
    """

    @pytest.fixture(params=resources_with_actions.keys())
    def resource(self, request) -> str:
        return request.param

    @pytest.fixture()
    def resource_client(self, client: Client, resource: str) -> ResourceActionsClient:
        """
        Extract the resource actions client from the client.
        """
        return getattr(client, resource).actions

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        resource_client: ResourceActionsClient,
        resource: str,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url=f"/{resource}/actions/1",
        )

        assert_bound_action1(action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "params",
        [
            {},
            {"status": ["running"], "sort": ["status"], "page": 2, "per_page": 10},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: ResourceActionsClient,
        resource: str,
        action_list_response,
        params,
    ):
        request_mock.return_value = action_list_response

        result = resource_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url=f"/{resource}/actions",
            params=params,
        )

        assert result.meta is not None

        actions = result.actions
        assert len(actions) == 2
        assert_bound_action1(actions[0], resource_client._parent.actions)
        assert_bound_action2(actions[1], resource_client._parent.actions)

    @pytest.mark.parametrize(
        "params",
        [
            {},
            {"status": ["running"], "sort": ["status"]},
        ],
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        resource_client: ResourceActionsClient,
        resource: str,
        action_list_response,
        params,
    ):
        request_mock.return_value = action_list_response

        actions = resource_client.get_all(**params)

        request_mock.assert_called_with(
            method="GET",
            url=f"/{resource}/actions",
            params={**params, "page": 1, "per_page": 50},
        )

        assert len(actions) == 2
        assert_bound_action1(actions[0], resource_client._parent.actions)
        assert_bound_action2(actions[1], resource_client._parent.actions)


class TestResourceObjectActionsClient:
    """
    /<resource>/<id>/actions
    """

    @pytest.fixture(params=resources_with_actions.keys())
    def resource(self, request):
        return request.param

    @pytest.fixture()
    def resource_client(self, client: Client, resource: str) -> ResourceClientBase:
        return getattr(client, resource)

    @pytest.fixture()
    def bound_model(self, client: Client, resource: str) -> BoundModelBase:
        _, bound_model_class = resources_with_actions[resource]
        resource_client = getattr(client, resource)
        return bound_model_class(resource_client, data={"id": 1})

    @pytest.mark.parametrize(
        "params",
        [
            {},
            {"status": ["running"], "sort": ["status"], "page": 2, "per_page": 10},
        ],
    )
    def test_get_actions_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: ResourceClientBase,
        resource: str,
        bound_model: BoundModelBase,
        action_list_response,
        params,
    ):
        request_mock.return_value = action_list_response

        result = resource_client.get_actions_list(bound_model, **params)

        request_mock.assert_called_with(
            method="GET",
            url=f"/{resource}/1/actions",
            params=params,
        )

        assert result.meta is not None

        actions = result.actions
        assert len(actions) == 2
        assert_bound_action1(actions[0], resource_client._parent.actions)
        assert_bound_action2(actions[1], resource_client._parent.actions)

    @pytest.mark.parametrize(
        "params",
        [
            {},
            {"status": ["running"], "sort": ["status"]},
        ],
    )
    def test_get_actions(
        self,
        request_mock: mock.MagicMock,
        resource_client: ResourceClientBase,
        resource: str,
        bound_model: BoundModelBase,
        action_list_response,
        params,
    ):
        request_mock.return_value = action_list_response

        actions = resource_client.get_actions(bound_model, **params)

        request_mock.assert_called_with(
            method="GET",
            url=f"/{resource}/1/actions",
            params={**params, "page": 1, "per_page": 50},
        )

        assert len(actions) == 2
        assert_bound_action1(actions[0], resource_client._parent.actions)
        assert_bound_action2(actions[1], resource_client._parent.actions)


class TestBoundModelActions:
    """
    /<resource>/<id>/actions
    """

    @pytest.fixture(params=resources_with_actions.keys())
    def resource(self, request):
        return request.param

    @pytest.fixture()
    def bound_model(self, client: Client, resource: str) -> ResourceClientBase:
        _, bound_model_class = resources_with_actions[resource]
        resource_client = getattr(client, resource)
        return bound_model_class(resource_client, data={"id": 1})

    @pytest.mark.parametrize(
        "params",
        [
            {},
            {"status": ["running"], "sort": ["status"], "page": 2, "per_page": 10},
        ],
    )
    def test_get_actions_list(
        self,
        request_mock: mock.MagicMock,
        bound_model: BoundModelBase,
        resource: str,
        action_list_response,
        params,
    ):
        request_mock.return_value = action_list_response

        result = bound_model.get_actions_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url=f"/{resource}/1/actions",
            params=params,
        )

        assert result.meta is not None

        actions = result.actions
        assert len(actions) == 2
        assert_bound_action1(actions[0], bound_model._client._parent.actions)
        assert_bound_action2(actions[1], bound_model._client._parent.actions)

    @pytest.mark.parametrize(
        "params",
        [
            {},
            {"status": ["running"], "sort": ["status"]},
        ],
    )
    def test_get_actions(
        self,
        request_mock: mock.MagicMock,
        bound_model: BoundModelBase,
        resource: str,
        action_list_response,
        params,
    ):
        request_mock.return_value = action_list_response

        actions = bound_model.get_actions(**params)

        request_mock.assert_called_with(
            method="GET",
            url=f"/{resource}/1/actions",
            params={**params, "page": 1, "per_page": 50},
        )

        assert len(actions) == 2
        assert_bound_action1(actions[0], bound_model._client._parent.actions)
        assert_bound_action2(actions[1], bound_model._client._parent.actions)


class TestActionsClient:
    @pytest.fixture()
    def actions_client(self, client: Client) -> ActionsClient:
        return client.actions

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        actions_client: ActionsClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = actions_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/actions/1",
        )
        assert_bound_action1(action, actions_client)

    @pytest.mark.parametrize(
        "params",
        [
            {},
            {"status": ["running"], "sort": ["status"], "page": 2, "per_page": 10},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        actions_client: ActionsClient,
        action_list_response,
        params,
    ):
        request_mock.return_value = action_list_response

        with pytest.deprecated_call():
            result = actions_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/actions",
            params=params,
        )

        assert result.meta is not None

        actions = result.actions
        assert len(actions) == 2
        assert_bound_action1(actions[0], actions_client)
        assert_bound_action2(actions[1], actions_client)

    @pytest.mark.parametrize(
        "params",
        [
            {},
            {"status": ["running"], "sort": ["status"]},
        ],
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        actions_client: ActionsClient,
        action_list_response,
        params,
    ):
        request_mock.return_value = action_list_response

        with pytest.deprecated_call():
            actions = actions_client.get_all(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/actions",
            params={**params, "page": 1, "per_page": 50},
        )

        assert len(actions) == 2
        assert_bound_action1(actions[0], actions_client)
        assert_bound_action2(actions[1], actions_client)



from __future__ import annotations

import pytest


@pytest.fixture()
def zone1():
    return {
        "id": 42,
        "name": "example1.com",
        "created": "2016-01-30T23:55:00+00:00",
        "mode": "primary",
        "ttl": 10800,
        "protection": {
            "delete": False,
        },
        "labels": {
            "key": "value",
        },
        "primary_nameservers": [
            {"address": "198.51.100.1", "port": 53},
            {"address": "203.0.113.1", "port": 53},
        ],
        "record_count": 0,
        "status": "ok",
        "registrar": "hetzner",
        "authoritative_nameservers": {
            "assigned": [
                "hydrogen.ns.hetzner.com.",
                "oxygen.ns.hetzner.com.",
                "helium.ns.hetzner.de.",
            ],
            "delegated": [
                "hydrogen.ns.hetzner.com.",
                "oxygen.ns.hetzner.com.",
                "helium.ns.hetzner.de.",
            ],
            "delegation_last_check": "2016-01-30T23:55:00+00:00",
            "delegation_status": "valid",
        },
    }


@pytest.fixture()
def zone2():
    return {
        "id": 43,
        "name": "example2.com",
        "created": "2016-01-30T23:55:00+00:00",
        "mode": "secondary",
        "ttl": 10800,
        "protection": {
            "delete": False,
        },
        "labels": {
            "key": "value",
        },
        "primary_nameservers": [
            {"address": "198.51.100.1", "port": 53},
            {"address": "203.0.113.1", "port": 53},
        ],
        "record_count": 0,
        "status": "ok",
        "registrar": "hetzner",
        "authoritative_nameservers": {
            "assigned": [],
            "delegated": [
                "hydrogen.ns.hetzner.com.",
                "oxygen.ns.hetzner.com.",
                "helium.ns.hetzner.de.",
            ],
            "delegation_last_check": "2016-01-30T23:55:00+00:00",
            "delegation_status": "valid",
        },
    }


@pytest.fixture()
def zone_rrset1():
    return {
        "zone": 42,
        "id": "www/A",
        "name": "www",
        "type": "A",
        "ttl": 3600,
        "labels": {"key": "value"},
        "protection": {"change": False},
        "records": [
            {"value": "198.51.100.1", "comment": "web server"},
        ],
    }


@pytest.fixture()
def zone_rrset2():
    return {
        "zone": 42,
        "id": "blog/A",
        "name": "blog",
        "type": "A",
        "ttl": 3600,
        "labels": {"key": "value"},
        "protection": {"change": False},
        "records": [
            {"value": "198.51.100.1", "comment": "web server"},
        ],
    }


@pytest.fixture()
def zone_response(zone1):
    return {"zone": zone1}


@pytest.fixture()
def zone_list_response(zone1, zone2):
    return {
        "zones": [zone1, zone2],
    }


@pytest.fixture()
def zone_create_response(zone1, action1_running):
    return {
        "zone": zone1,
        "action": action1_running,
    }


@pytest.fixture()
def zone_rrset_response(zone_rrset1):
    return {
        "rrset": zone_rrset1,
    }


@pytest.fixture()
def zone_rrset_list_response(zone_rrset1, zone_rrset2):
    return {
        "rrsets": [zone_rrset1, zone_rrset2],
    }


@pytest.fixture()
def zone_rrset_create_response(zone_rrset1, action1_running):
    return {
        "rrset": zone_rrset1,
        "action": action1_running,
    }



# pylint: disable=redefined-outer-name

from __future__ import annotations

import inspect
import warnings
from collections.abc import Callable
from typing import ClassVar, TypedDict
from unittest import mock

import pytest

from hcloud import Client
from hcloud.actions import ActionsClient, BoundAction


@pytest.fixture(autouse=True, scope="session")
def patch_package_version():
    with mock.patch("hcloud._client.__version__", "0.0.0"):
        yield


@pytest.fixture()
def request_mock() -> mock.MagicMock:
    return mock.MagicMock()


@pytest.fixture()
def client(request_mock) -> Client:
    c = Client(
        token="TOKEN",
        # Speed up tests that use `_poll_interval_func`
        poll_interval=0.0,
        poll_max_retries=3,
    )
    c._client.request = request_mock
    c._client_hetzner.request = request_mock
    return c


def assert_bound_action1(o: BoundAction, client: ActionsClient):
    assert o.id == 1
    assert o.command == "command"
    assert o._client == client


def assert_bound_action2(o: BoundAction, client: ActionsClient):
    assert o.id == 2
    assert o.command == "command"
    assert o._client == client


@pytest.fixture()
def action1_running():
    return {
        "id": 1,
        "command": "command",
        "status": "running",
        "progress": 0,
        "started": "2016-01-30T23:50+00:00",
        "finished": None,
        "resources": [{"id": 42, "type": "resource"}],
        "error": None,
    }


@pytest.fixture()
def action2_running():
    return {
        "id": 2,
        "command": "command",
        "status": "running",
        "progress": 20,
        "started": "2016-01-30T23:50+00:00",
        "finished": None,
        "resources": [{"id": 43, "type": "resource"}],
        "error": None,
    }


@pytest.fixture()
def action1_success(action1_running):
    return {
        **action1_running,
        "status": "success",
        "progress": 100,
        "finished": "2016-01-31T00:10+00:00",
    }


@pytest.fixture()
def action2_success(action2_running):
    return {
        **action2_running,
        "status": "success",
        "progress": 100,
        "finished": "2016-01-31T00:10+00:00",
    }


@pytest.fixture()
def action1_error(action1_running):
    return {
        **action1_running,
        "status": "error",
        "progress": 100,
        "finished": "2016-01-31T00:10+00:00",
        "error": {"code": "action_failed", "message": "Action failed"},
    }


@pytest.fixture()
def action2_error(action2_running):
    return {
        **action2_running,
        "status": "error",
        "progress": 100,
        "finished": "2016-01-31T00:10+00:00",
        "error": {"code": "action_failed", "message": "Action failed"},
    }


@pytest.fixture()
def action_response(action1_running):
    return {
        "action": action1_running,
    }


@pytest.fixture()
def action_list_response(action1_running, action2_running):
    return {
        "actions": [
            action1_running,
            action2_running,
        ],
    }


def build_kwargs_mock(func: Callable) -> dict[str, mock.Mock]:
    """
    Generate a kwargs dict that may be passed to the provided function for testing purposes.
    """
    s = inspect.signature(func)

    kwargs = {}
    for name, param in s.parameters.items():
        if name in ("self",):
            continue

        if param.kind in (param.POSITIONAL_OR_KEYWORD, param.KEYWORD_ONLY):
            kwargs[name] = mock.Mock()
            continue

        # Ignore **kwargs
        if param.kind in (param.VAR_KEYWORD,):
            continue

        raise NotImplementedError(f"unsupported parameter kind: {param.kind}")

    return kwargs


def pytest_generate_tests(metafunc: pytest.Metafunc):
    """
    Magic function to generate a test for each bound model method.
    """
    if "bound_model_method" in metafunc.fixturenames:
        metafunc.parametrize("bound_model_method", metafunc.cls.methods)


class BoundModelTestOptions(TypedDict):
    sub_resource: bool
    client_method: str


class BoundModelTestCase:
    methods: ClassVar[list[Callable | tuple[Callable, BoundModelTestOptions]]]

    def test_method_list(self, bound_model):
        """
        Ensure the list of bound model methods is up to date.
        """
        # Unpack methods
        methods = [m[0] if isinstance(m, tuple) else m for m in self.__class__.methods]

        members_count = 0
        members_missing = []

        with warnings.catch_warnings():
            warnings.filterwarnings("ignore", category=DeprecationWarning)

            members = inspect.getmembers(
                bound_model,
                lambda m: inspect.ismethod(m)
                and m.__func__ in bound_model.__class__.__dict__.values(),
            )

        for name, member in members:
            # Ignore private methods
            if name.startswith("_"):
                continue

            # Actions methods are already tested in TestBoundModelActions.
            if name in ("__init__", "get_actions", "get_actions_list"):
                continue

            if member.__func__ in methods:
                members_count += 1
            else:
                members_missing.append(member.__func__.__qualname__)

        assert not members_missing, "untested methods:\n" + ",\n".join(members_missing)
        assert members_count == len(self.__class__.methods)

    def test_method(
        self,
        resource_client,
        bound_model,
        bound_model_method: Callable | tuple[Callable, BoundModelTestOptions],
    ):
        options = BoundModelTestOptions()
        if isinstance(bound_model_method, tuple):
            bound_model_method, options = bound_model_method

        resource_client_method_name = options.get(
            "client_method",
            bound_model_method.__name__,
        )

        # Check if the resource client has a method named after the bound model method.
        assert hasattr(resource_client, resource_client_method_name)

        # Mock the resource client method.
        resource_client_method_mock = mock.MagicMock()
        setattr(
            resource_client,
            resource_client_method_name,
            resource_client_method_mock,
        )

        kwargs = build_kwargs_mock(bound_model_method)

        # Call the bound model method
        result = getattr(bound_model, bound_model_method.__name__)(**kwargs)

        if options.get("sub_resource"):
            resource_client_method_mock.assert_called_with(**kwargs)
        else:
            resource_client_method_mock.assert_called_with(bound_model, **kwargs)

        assert result is resource_client_method_mock.return_value



from __future__ import annotations

import pytest

from hcloud.exp.zone import format_txt_record, is_txt_record_quoted


@pytest.mark.parametrize(
    ("value", "expected"),
    [
        ("hello world", False),
        ('"hello world', False),
        ('"hello world"', True),
    ],
)
def test_is_txt_record_quoted(value: str, expected: bool):
    assert is_txt_record_quoted(value) == expected


MANY_A = "a" * 255
SOME_B = "b" * 10


@pytest.mark.parametrize(
    ("value", "expected"),
    [
        ("", ""),
        ('""', '"\\"\\""'),
        ("hello world", '"hello world"'),
        ("hello\nworld", '"hello\nworld"'),
        ('hello "world"', '"hello \\"world\\""'),
        ('hello "world', '"hello \\"world"'),
        (MANY_A + SOME_B, f'"{MANY_A}" "{SOME_B}"'),
    ],
)
def test_format_txt_record(value: str, expected: str):
    assert format_txt_record(value) == expected



# pylint: disable=protected-access

from __future__ import annotations

from unittest import mock

import pytest
from dateutil.parser import isoparse

from hcloud import Client
from hcloud.locations import Location
from hcloud.storage_box_types import StorageBoxType
from hcloud.storage_boxes import (
    BoundStorageBox,
    BoundStorageBoxSnapshot,
    BoundStorageBoxSubaccount,
    StorageBox,
    StorageBoxAccessSettings,
    StorageBoxesClient,
    StorageBoxSnapshot,
    StorageBoxSnapshotPlan,
    StorageBoxSubaccount,
    StorageBoxSubaccountAccessSettings,
)

from ..conftest import BoundModelTestCase, assert_bound_action1


def assert_bound_storage_box(
    o: BoundStorageBox,
    resource_client: StorageBoxesClient,
):
    assert isinstance(o, BoundStorageBox)
    assert o._client is resource_client
    assert o.id == 42
    assert o.name == "storage-box1"


def assert_bound_storage_box_snapshot(
    o: BoundStorageBoxSnapshot,
    resource_client: StorageBoxesClient,
):
    assert isinstance(o, BoundStorageBoxSnapshot)
    assert o._client is resource_client
    assert o.id == 34
    assert o.name == "storage-box-snapshot1"


def assert_bound_storage_box_subaccount(
    o: BoundStorageBoxSubaccount,
    resource_client: StorageBoxesClient,
):
    assert isinstance(o, BoundStorageBoxSubaccount)
    assert o._client is resource_client
    assert o.id == 45
    assert o.username == "u42-sub1"


class TestBoundStorageBox(BoundModelTestCase):
    methods = [
        BoundStorageBox.update,
        BoundStorageBox.delete,
        BoundStorageBox.get_folders,
        BoundStorageBox.change_protection,
        BoundStorageBox.change_type,
        BoundStorageBox.disable_snapshot_plan,
        BoundStorageBox.enable_snapshot_plan,
        BoundStorageBox.reset_password,
        BoundStorageBox.rollback_snapshot,
        BoundStorageBox.update_access_settings,
        # Snapshots
        BoundStorageBox.create_snapshot,
        BoundStorageBox.get_snapshot_all,
        BoundStorageBox.get_snapshot_by_id,
        BoundStorageBox.get_snapshot_by_name,
        BoundStorageBox.get_snapshot_list,
        # Subaccounts
        BoundStorageBox.create_subaccount,
        BoundStorageBox.get_subaccount_all,
        BoundStorageBox.get_subaccount_by_id,
        BoundStorageBox.get_subaccount_by_name,
        BoundStorageBox.get_subaccount_by_username,
        BoundStorageBox.get_subaccount_list,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client) -> StorageBoxesClient:
        return client.storage_boxes

    @pytest.fixture()
    def bound_model(
        self,
        resource_client: StorageBoxesClient,
        storage_box1,
    ) -> BoundStorageBox:
        return BoundStorageBox(resource_client, data=storage_box1)

    def test_init(self, bound_model: BoundStorageBox, resource_client):
        o = bound_model

        assert_bound_storage_box(o, resource_client)

        assert o.storage_box_type.id == 42
        assert o.storage_box_type.name == "bx11"
        assert o.location.id == 1
        assert o.location.name == "fsn1"
        assert o.system == "FSN1-BX355"
        assert o.server == "u1337.your-storagebox.de"
        assert o.username == "u12345"
        assert o.labels == {"key": "value"}
        assert o.protection == {"delete": False}
        assert o.snapshot_plan.max_snapshots == 20
        assert o.snapshot_plan.minute == 0
        assert o.snapshot_plan.hour == 7
        assert o.snapshot_plan.day_of_week == 7
        assert o.snapshot_plan.day_of_month is None
        assert o.access_settings.reachable_externally is False
        assert o.access_settings.samba_enabled is False
        assert o.access_settings.ssh_enabled is False
        assert o.access_settings.webdav_enabled is False
        assert o.access_settings.zfs_enabled is False
        assert o.stats.size == 2342236717056
        assert o.stats.size_data == 2102612983808
        assert o.stats.size_snapshots == 239623733248
        assert o.status == "active"
        assert o.created == isoparse("2025-01-30T23:55:00Z")


class TestBoundStorageBoxSnapshot(BoundModelTestCase):
    methods = [
        (BoundStorageBoxSnapshot.update, {"client_method": "update_snapshot"}),
        (BoundStorageBoxSnapshot.delete, {"client_method": "delete_snapshot"}),
    ]

    @pytest.fixture()
    def resource_client(self, client: Client) -> StorageBoxesClient:
        return client.storage_boxes

    @pytest.fixture()
    def bound_model(
        self,
        resource_client: StorageBoxesClient,
        storage_box_snapshot1,
    ) -> BoundStorageBoxSnapshot:
        return BoundStorageBoxSnapshot(resource_client, data=storage_box_snapshot1)

    def test_init(self, bound_model: BoundStorageBoxSnapshot, resource_client):
        o = bound_model

        assert_bound_storage_box_snapshot(o, resource_client)

        assert isinstance(o.storage_box, BoundStorageBox)
        assert o.storage_box.id == 42

        assert o.description == ""
        assert o.is_automatic is False
        assert o.labels == {"key": "value"}
        assert o.stats.size == 394957594
        assert o.stats.size_filesystem == 3949572745
        assert o.created == isoparse("2025-11-10T19:16:57Z")

    def test_reload(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_snapshot1,
    ):
        o = BoundStorageBoxSnapshot(resource_client, data={"id": 34, "storage_box": 42})

        request_mock.return_value = {"snapshot": storage_box_snapshot1}

        o.reload()

        request_mock.assert_called_with(
            method="GET",
            url="/storage_boxes/42/snapshots/34",
        )

        assert o.labels is not None


class TestBoundStorageBoxSubaccount(BoundModelTestCase):
    methods = [
        (
            BoundStorageBoxSubaccount.update,
            {"client_method": "update_subaccount"},
        ),
        (
            BoundStorageBoxSubaccount.delete,
            {"client_method": "delete_subaccount"},
        ),
        (
            BoundStorageBoxSubaccount.change_home_directory,
            {"client_method": "change_subaccount_home_directory"},
        ),
        (
            BoundStorageBoxSubaccount.reset_password,
            {"client_method": "reset_subaccount_password"},
        ),
        (
            BoundStorageBoxSubaccount.update_access_settings,
            {"client_method": "update_subaccount_access_settings"},
        ),
    ]

    @pytest.fixture()
    def resource_client(self, client: Client) -> StorageBoxesClient:
        return client.storage_boxes

    @pytest.fixture()
    def bound_model(
        self,
        resource_client: StorageBoxesClient,
        storage_box_subaccount1,
    ) -> BoundStorageBoxSubaccount:
        return BoundStorageBoxSubaccount(resource_client, data=storage_box_subaccount1)

    def test_init(self, bound_model: BoundStorageBoxSubaccount, resource_client):
        o = bound_model

        assert_bound_storage_box_subaccount(o, resource_client)

        assert isinstance(o.storage_box, BoundStorageBox)
        assert o.storage_box.id == 42

        assert o.username == "u42-sub1"
        assert o.description == "Required by foo"
        assert o.server == "u42-sub1.your-storagebox.de"
        assert o.home_directory == "tmp/"
        assert o.access_settings.reachable_externally is True
        assert o.access_settings.samba_enabled is False
        assert o.access_settings.ssh_enabled is True
        assert o.access_settings.webdav_enabled is False
        assert o.access_settings.readonly is False
        assert o.labels == {"key": "value"}
        assert o.created == isoparse("2025-11-10T19:18:57Z")

    def test_reload(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_subaccount1,
    ):
        o = BoundStorageBoxSubaccount(
            resource_client, data={"id": 45, "storage_box": 42}
        )

        request_mock.return_value = {"subaccount": storage_box_subaccount1}

        o.reload()

        request_mock.assert_called_with(
            method="GET",
            url="/storage_boxes/42/subaccounts/45",
        )

        assert o.labels is not None


class TestStorageBoxClient:
    @pytest.fixture()
    def resource_client(self, client: Client) -> StorageBoxesClient:
        return client.storage_boxes

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box1,
    ):
        request_mock.return_value = {"storage_box": storage_box1}

        result = resource_client.get_by_id(42)

        request_mock.assert_called_with(
            method="GET",
            url="/storage_boxes/42",
        )

        assert_bound_storage_box(result, resource_client)

    @pytest.mark.parametrize(
        "params",
        [
            {"name": "storage-box1"},
            {"label_selector": "key=value"},
            {"page": 1, "per_page": 10},
            {"sort": ["id:asc"]},
            {},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box1,
        storage_box2,
        params,
    ):
        request_mock.return_value = {"storage_boxes": [storage_box1, storage_box2]}

        result = resource_client.get_list(**params)

        request_mock.assert_called_with(
            url="/storage_boxes",
            method="GET",
            params=params,
        )

        assert result.meta is not None
        assert len(result.storage_boxes) == 2

        result1 = result.storage_boxes[0]
        result2 = result.storage_boxes[1]

        assert result1._client is resource_client
        assert result1.id == 42
        assert result1.name == "storage-box1"

        assert result2._client is resource_client
        assert result2.id == 43
        assert result2.name == "storage-box2"

    @pytest.mark.parametrize(
        "params",
        [
            {"name": "storage-box1"},
            {"label_selector": "key=value"},
            {"sort": ["id:asc"]},
            {},
        ],
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box1,
        storage_box2,
        params,
    ):
        request_mock.return_value = {"storage_boxes": [storage_box1, storage_box2]}

        result = resource_client.get_all(**params)

        request_mock.assert_called_with(
            url="/storage_boxes",
            method="GET",
            params={**params, "page": 1, "per_page": 50},
        )

        assert len(result) == 2

        result1 = result[0]
        result2 = result[1]

        assert result1._client is resource_client
        assert result1.id == 42
        assert result1.name == "storage-box1"

        assert result2._client is resource_client
        assert result2.id == 43
        assert result2.name == "storage-box2"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box1,
    ):
        request_mock.return_value = {"storage_boxes": [storage_box1]}

        result = resource_client.get_by_name("bx11")

        params = {"name": "bx11"}

        request_mock.assert_called_with(
            method="GET",
            url="/storage_boxes",
            params=params,
        )

        assert_bound_storage_box(result, resource_client)

    def test_create(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box1,
        action1_running,
    ):
        request_mock.return_value = {
            "storage_box": storage_box1,
            "action": action1_running,
        }

        result = resource_client.create(
            name="storage-box1",
            password="secret-password",
            location=Location(name="fsn1"),
            storage_box_type=StorageBoxType(name="bx11"),
            ssh_keys=[],
            access_settings=StorageBoxAccessSettings(
                reachable_externally=True,
                ssh_enabled=True,
                samba_enabled=False,
            ),
            labels={"key": "value"},
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes",
            json={
                "name": "storage-box1",
                "password": "secret-password",
                "location": "fsn1",
                "storage_box_type": "bx11",
                "ssh_keys": [],
                "access_settings": {
                    "reachable_externally": True,
                    "samba_enabled": False,
                    "ssh_enabled": True,
                },
                "labels": {"key": "value"},
            },
        )

        assert_bound_storage_box(result.storage_box, resource_client)

    def test_update(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box1,
    ):
        request_mock.return_value = {
            "storage_box": storage_box1,
        }

        result = resource_client.update(
            StorageBox(id=42),
            name="name",
            labels={"key": "value"},
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/storage_boxes/42",
            json={
                "name": "name",
                "labels": {"key": "value"},
            },
        )

        assert_bound_storage_box(result, resource_client)

    def test_delete(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action1_running,
    ):
        request_mock.return_value = {
            "action": action1_running,
        }

        result = resource_client.delete(StorageBox(id=42))

        request_mock.assert_called_with(
            method="DELETE",
            url="/storage_boxes/42",
        )

        assert_bound_action1(result.action, resource_client._parent.actions)

    @pytest.mark.parametrize(
        "params",
        [
            {"path": "dir1/path"},
            {},
        ],
    )
    def test_get_folders(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        params,
    ):
        request_mock.return_value = {
            "folders": ["dir1", "dir2"],
        }

        result = resource_client.get_folders(StorageBox(id=42), **params)

        request_mock.assert_called_with(
            method="GET", url="/storage_boxes/42/folders", params=params
        )

        assert result.folders == ["dir1", "dir2"]

    def test_change_protection(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.change_protection(StorageBox(id=42), delete=True)

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/actions/change_protection",
            json={"delete": True},
        )

        assert_bound_action1(action, resource_client._parent.actions)

    def test_change_type(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.change_type(
            StorageBox(id=42),
            StorageBoxType(name="bx21"),
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/actions/change_type",
            json={"storage_box_type": "bx21"},
        )

        assert_bound_action1(action, resource_client._parent.actions)

    def test_reset_password(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.reset_password(
            StorageBox(id=42),
            password="password",
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/actions/reset_password",
            json={"password": "password"},
        )

        assert_bound_action1(action, resource_client._parent.actions)

    def test_update_access_settings(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.update_access_settings(
            StorageBox(id=42),
            StorageBoxAccessSettings(
                reachable_externally=True,
                ssh_enabled=True,
                webdav_enabled=False,
            ),
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/actions/update_access_settings",
            json={
                "reachable_externally": True,
                "ssh_enabled": True,
                "webdav_enabled": False,
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)

    def test_rollback_snapshot(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.rollback_snapshot(
            StorageBox(id=42),
            StorageBoxSnapshot(id=32),
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/actions/rollback_snapshot",
            json={"snapshot": 32},
        )

        assert_bound_action1(action, resource_client._parent.actions)

    def test_disable_snapshot_plan(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.disable_snapshot_plan(
            StorageBox(id=42),
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/actions/disable_snapshot_plan",
        )

        assert_bound_action1(action, resource_client._parent.actions)

    def test_enable_snapshot_plan(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.enable_snapshot_plan(
            StorageBox(id=42),
            StorageBoxSnapshotPlan(
                max_snapshots=10,
                hour=3,
                minute=30,
                day_of_week=None,
            ),
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/actions/enable_snapshot_plan",
            json={
                "max_snapshots": 10,
                "hour": 3,
                "minute": 30,
                "day_of_week": None,
                "day_of_month": None,
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)

    # Snapshots
    ###########################################################################

    def test_get_snapshot_by_id(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_snapshot1,
    ):
        request_mock.return_value = {"snapshot": storage_box_snapshot1}

        result = resource_client.get_snapshot_by_id(StorageBox(42), 34)

        request_mock.assert_called_with(
            method="GET",
            url="/storage_boxes/42/snapshots/34",
        )

        assert_bound_storage_box_snapshot(result, resource_client)

    @pytest.mark.parametrize(
        "params",
        [
            {"name": "storage-box-snapshot1"},
            {"is_automatic": True},
            {"label_selector": "key=value"},
            # {"page": 1, "per_page": 10}  # No pagination
            {"sort": ["id:asc"]},
            {},
        ],
    )
    def test_get_snapshot_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_snapshot1,
        storage_box_snapshot2,
        params,
    ):
        request_mock.return_value = {
            "snapshots": [storage_box_snapshot1, storage_box_snapshot2]
        }

        result = resource_client.get_snapshot_list(StorageBox(42), **params)

        request_mock.assert_called_with(
            url="/storage_boxes/42/snapshots",
            method="GET",
            params=params,
        )

        assert result.meta is not None
        assert len(result.snapshots) == 2

        result1 = result.snapshots[0]
        result2 = result.snapshots[1]

        assert result1._client is resource_client
        assert result1.id == 34
        assert result1.name == "storage-box-snapshot1"
        assert isinstance(result1.storage_box, BoundStorageBox)
        assert result1.storage_box.id == 42

        assert result2._client is resource_client
        assert result2.id == 35
        assert result2.name == "storage-box-snapshot2"
        assert isinstance(result2.storage_box, BoundStorageBox)
        assert result2.storage_box.id == 42

    @pytest.mark.parametrize(
        "params",
        [
            {"name": "storage-box-snapshot1"},
            {"is_automatic": True},
            {"label_selector": "key=value"},
            {"sort": ["id:asc"]},
            {},
        ],
    )
    def test_get_snapshot_all(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_snapshot1,
        storage_box_snapshot2,
        params,
    ):
        request_mock.return_value = {
            "snapshots": [storage_box_snapshot1, storage_box_snapshot2]
        }

        result = resource_client.get_snapshot_all(StorageBox(42), **params)

        request_mock.assert_called_with(
            url="/storage_boxes/42/snapshots",
            method="GET",
            params=params,
        )

        assert len(result) == 2

        result1 = result[0]
        result2 = result[1]

        assert result1._client is resource_client
        assert result1.id == 34
        assert result1.name == "storage-box-snapshot1"
        assert isinstance(result1.storage_box, BoundStorageBox)
        assert result1.storage_box.id == 42

        assert result2._client is resource_client
        assert result2.id == 35
        assert result2.name == "storage-box-snapshot2"
        assert isinstance(result2.storage_box, BoundStorageBox)
        assert result2.storage_box.id == 42

    def test_get_snapshot_by_name(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_snapshot1,
    ):
        request_mock.return_value = {"snapshots": [storage_box_snapshot1]}

        result = resource_client.get_snapshot_by_name(
            StorageBox(42), "storage-box-snapshot1"
        )

        request_mock.assert_called_with(
            method="GET",
            url="/storage_boxes/42/snapshots",
            params={"name": "storage-box-snapshot1"},
        )

        assert_bound_storage_box_snapshot(result, resource_client)

    def test_create_snapshot(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_snapshot1: dict,
        action1_running,
    ):
        request_mock.return_value = {
            "snapshot": {
                # Only a partial object is returned
                key: storage_box_snapshot1[key]
                for key in ["id", "storage_box"]
            },
            "action": action1_running,
        }

        result = resource_client.create_snapshot(
            StorageBox(42),
            description="something",
            labels={"key": "value"},
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/snapshots",
            json={
                "description": "something",
                "labels": {"key": "value"},
            },
        )

        assert isinstance(result.snapshot, BoundStorageBoxSnapshot)
        assert result.snapshot._client is resource_client
        assert result.snapshot.id == 34

        assert_bound_action1(result.action, resource_client._parent.actions)

    def test_update_snapshot(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_snapshot1,
    ):
        request_mock.return_value = {
            "snapshot": storage_box_snapshot1,
        }

        result = resource_client.update_snapshot(
            StorageBoxSnapshot(id=34, storage_box=StorageBox(42)),
            description="something",
            labels={"key": "value"},
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/storage_boxes/42/snapshots/34",
            json={
                "description": "something",
                "labels": {"key": "value"},
            },
        )

        assert_bound_storage_box_snapshot(result, resource_client)

    def test_delete_snapshot(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action1_running,
    ):
        request_mock.return_value = {
            "action": action1_running,
        }

        result = resource_client.delete_snapshot(
            StorageBoxSnapshot(id=34, storage_box=StorageBox(42))
        )

        request_mock.assert_called_with(
            method="DELETE",
            url="/storage_boxes/42/snapshots/34",
        )

        assert_bound_action1(result.action, resource_client._parent.actions)

    # Subaccounts
    ###########################################################################

    def test_get_subaccount_by_id(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_subaccount1,
    ):
        request_mock.return_value = {"subaccount": storage_box_subaccount1}

        result = resource_client.get_subaccount_by_id(StorageBox(42), 45)

        request_mock.assert_called_with(
            method="GET",
            url="/storage_boxes/42/subaccounts/45",
        )

        assert_bound_storage_box_subaccount(result, resource_client)

    def test_get_subaccount_by_name(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_subaccount1,
    ):
        request_mock.return_value = {"subaccounts": [storage_box_subaccount1]}

        result = resource_client.get_subaccount_by_name(StorageBox(42), "subaccount1")

        request_mock.assert_called_with(
            method="GET",
            url="/storage_boxes/42/subaccounts",
            params={"name": "subaccount1"},
        )

        assert_bound_storage_box_subaccount(result, resource_client)

    @pytest.mark.parametrize(
        "params",
        [
            {"name": "subaccount1"},
            {"username": "u42-sub1"},
            {"label_selector": "key=value"},
            # {"page": 1, "per_page": 10}  # No pagination
            {"sort": ["id:asc"]},
            {},
        ],
    )
    def test_get_subaccount_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_subaccount1,
        storage_box_subaccount2,
        params,
    ):
        request_mock.return_value = {
            "subaccounts": [storage_box_subaccount1, storage_box_subaccount2]
        }

        result = resource_client.get_subaccount_list(StorageBox(42), **params)

        request_mock.assert_called_with(
            url="/storage_boxes/42/subaccounts",
            method="GET",
            params=params,
        )

        assert result.meta is not None
        assert len(result.subaccounts) == 2

        result1 = result.subaccounts[0]
        result2 = result.subaccounts[1]

        assert result1._client is resource_client
        assert result1.id == 45
        assert result1.username == "u42-sub1"
        assert isinstance(result1.storage_box, BoundStorageBox)
        assert result1.storage_box.id == 42

        assert result2._client is resource_client
        assert result2.id == 46
        assert result2.username == "u42-sub2"
        assert isinstance(result2.storage_box, BoundStorageBox)
        assert result2.storage_box.id == 42

    @pytest.mark.parametrize(
        "params",
        [
            {"name": "subaccount1"},
            {"username": "u42-sub1"},
            {"label_selector": "key=value"},
            {"sort": ["id:asc"]},
            {},
        ],
    )
    def test_get_subaccount_all(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_subaccount1,
        storage_box_subaccount2,
        params,
    ):
        request_mock.return_value = {
            "subaccounts": [storage_box_subaccount1, storage_box_subaccount2]
        }

        result = resource_client.get_subaccount_all(StorageBox(42), **params)

        request_mock.assert_called_with(
            url="/storage_boxes/42/subaccounts",
            method="GET",
            params=params,
        )

        assert len(result) == 2

        result1 = result[0]
        result2 = result[1]

        assert result1._client is resource_client
        assert result1.id == 45
        assert result1.username == "u42-sub1"
        assert isinstance(result1.storage_box, BoundStorageBox)
        assert result1.storage_box.id == 42

        assert result2._client is resource_client
        assert result2.id == 46
        assert result2.username == "u42-sub2"
        assert isinstance(result2.storage_box, BoundStorageBox)
        assert result2.storage_box.id == 42

    def test_get_subaccount_by_username(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_subaccount1,
    ):
        request_mock.return_value = {"subaccounts": [storage_box_subaccount1]}

        result = resource_client.get_subaccount_by_username(StorageBox(42), "u42-sub1")

        request_mock.assert_called_with(
            method="GET",
            url="/storage_boxes/42/subaccounts",
            params={"username": "u42-sub1"},
        )

        assert_bound_storage_box_subaccount(result, resource_client)

    def test_create_subaccount(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_subaccount1: dict,
        action1_running,
    ):
        request_mock.return_value = {
            "subaccount": {
                # Only a partial object is returned
                key: storage_box_subaccount1[key]
                for key in ["id", "storage_box"]
            },
            "action": action1_running,
        }

        result = resource_client.create_subaccount(
            StorageBox(42),
            name="subaccount1",
            home_directory="tmp",
            password="secret",
            access_settings=StorageBoxSubaccountAccessSettings(
                reachable_externally=True,
                ssh_enabled=True,
                readonly=False,
            ),
            description="something",
            labels={"key": "value"},
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/subaccounts",
            json={
                "name": "subaccount1",
                "home_directory": "tmp",
                "password": "secret",
                "access_settings": {
                    "reachable_externally": True,
                    "ssh_enabled": True,
                    "readonly": False,
                },
                "description": "something",
                "labels": {"key": "value"},
            },
        )

        assert isinstance(result.subaccount, BoundStorageBoxSubaccount)
        assert result.subaccount._client is resource_client
        assert result.subaccount.id == 45

        assert_bound_action1(result.action, resource_client._parent.actions)

    def test_update_subaccount(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        storage_box_subaccount1,
    ):
        request_mock.return_value = {
            "subaccount": storage_box_subaccount1,
        }

        result = resource_client.update_subaccount(
            StorageBoxSubaccount(id=45, storage_box=StorageBox(42)),
            description="something",
            labels={"key": "value"},
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/storage_boxes/42/subaccounts/45",
            json={
                "description": "something",
                "labels": {"key": "value"},
            },
        )

        assert_bound_storage_box_subaccount(result, resource_client)

    def test_delete_subaccount(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action1_running,
    ):
        request_mock.return_value = {
            "action": action1_running,
        }

        result = resource_client.delete_subaccount(
            StorageBoxSubaccount(id=45, storage_box=StorageBox(42)),
        )

        request_mock.assert_called_with(
            method="DELETE",
            url="/storage_boxes/42/subaccounts/45",
        )

        assert_bound_action1(result.action, resource_client._parent.actions)

    def test_change_subaccount_home_directory(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.change_subaccount_home_directory(
            StorageBoxSubaccount(id=45, storage_box=StorageBox(42)),
            home_directory="path",
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/subaccounts/45/actions/change_home_directory",
            json={
                "home_directory": "path",
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)

    def test_reset_subaccount_password(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.reset_subaccount_password(
            StorageBoxSubaccount(id=45, storage_box=StorageBox(42)),
            password="password",
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/subaccounts/45/actions/reset_subaccount_password",
            json={
                "password": "password",
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)

    def test_update_subaccount_access_settings(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxesClient,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.update_subaccount_access_settings(
            StorageBoxSubaccount(id=45, storage_box=StorageBox(42)),
            access_settings=StorageBoxSubaccountAccessSettings(
                reachable_externally=True,
                ssh_enabled=True,
                samba_enabled=False,
            ),
        )

        request_mock.assert_called_with(
            method="POST",
            url="/storage_boxes/42/subaccounts/45/actions/update_access_settings",
            json={
                "reachable_externally": True,
                "ssh_enabled": True,
                "samba_enabled": False,
            },
        )

        assert_bound_action1(action, resource_client._parent.actions)



from __future__ import annotations

import pytest

from hcloud import (
    APIException,
    HCloudException,
)
from hcloud.actions import Action, ActionFailedException, ActionTimeoutException

running_action = Action(
    id=12345,
    command="action_command",
    status=Action.STATUS_RUNNING,
)

failed_action = Action(
    id=12345,
    command="action_command",
    status=Action.STATUS_ERROR,
    error={"code": "action_failed", "message": "Action failed"},
)


@pytest.mark.parametrize(
    ("exception", "expected"),
    [
        (
            # Should never be raised by itself
            HCloudException(),
            "",
        ),
        (
            # Should never be raised by itself
            HCloudException("A test error"),
            "A test error",
        ),
        (
            APIException(code="conflict", message="API error message", details=None),
            "API error message (conflict)",
        ),
        (
            APIException(
                code="conflict",
                message="API error message",
                details=None,
                correlation_id="fddea8fabd02fb21",
            ),
            "API error message (conflict, fddea8fabd02fb21)",
        ),
        (
            ActionFailedException(failed_action),
            "The pending action failed: Action failed (action_failed, 12345)",
        ),
        (
            ActionTimeoutException(running_action),
            "The pending action timed out (action_command, 12345)",
        ),
    ],
)
def test_exceptions(exception, expected):
    assert str(exception) == expected



from __future__ import annotations

import datetime
from datetime import timezone

import pytest

from hcloud.firewalls import (
    Firewall,
    FirewallResource,
    FirewallResourceAppliedToResources,
    FirewallResourceLabelSelector,
    FirewallRule,
)


@pytest.mark.parametrize(
    "value",
    [
        (Firewall(id=1),),
        (FirewallRule(direction="in", protocol="icmp", source_ips=[]),),
        (FirewallResource(type="server"),),
        (FirewallResourceAppliedToResources(type="server"),),
        (FirewallResourceLabelSelector(),),
    ],
)
def test_eq(value):
    assert value.__eq__(value)


class TestFirewall:
    def test_created_is_datetime(self):
        firewall = Firewall(id=1, created="2016-01-30T23:50+00:00")
        assert firewall.created == datetime.datetime(
            2016, 1, 30, 23, 50, tzinfo=timezone.utc
        )



from __future__ import annotations

import pytest


@pytest.fixture()
def storage_box_type1():
    return {
        "id": 42,
        "name": "bx11",
        "description": "BX11",
        "snapshot_limit": 10,
        "automatic_snapshot_limit": 10,
        "subaccounts_limit": 100,
        "size": 1099511627776,
        "prices": [
            {
                "location": "fsn1",
                "price_hourly": {"gross": "0.0051", "net": "0.0051"},
                "price_monthly": {"gross": "3.2000", "net": "3.2000"},
                "setup_fee": {"gross": "0.0000", "net": "0.0000"},
            }
        ],
        "deprecation": {
            "unavailable_after": "2023-09-01T00:00:00+00:00",
            "announced": "2023-06-01T00:00:00+00:00",
        },
    }


@pytest.fixture()
def storage_box_type2():
    return {
        "id": 43,
        "name": "bx21",
        "description": "BX21",
        "snapshot_limit": 20,
        "automatic_snapshot_limit": 20,
        "subaccounts_limit": 100,
        "size": 5497558138880,
        "prices": [
            {
                "location": "fsn1",
                "price_hourly": {"net": "1.0000", "gross": "1.1900"},
                "price_monthly": {"net": "1.0000", "gross": "1.1900"},
                "setup_fee": {"net": "1.0000", "gross": "1.1900"},
            }
        ],
        "deprecation": None,
    }



from __future__ import annotations

from unittest import mock

import pytest

from hcloud import Client
from hcloud.ssh_keys import BoundSSHKey, SSHKey, SSHKeysClient

from ..conftest import BoundModelTestCase


class TestBoundSSHKey(BoundModelTestCase):
    methods = [
        BoundSSHKey.update,
        BoundSSHKey.delete,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client) -> SSHKeysClient:
        return client.ssh_keys

    @pytest.fixture()
    def bound_model(self, resource_client: SSHKeysClient) -> BoundSSHKey:
        return BoundSSHKey(resource_client, data=dict(id=14))

    def test_init(self, ssh_key_response):
        bound_ssh_key = BoundSSHKey(
            client=mock.MagicMock(), data=ssh_key_response["ssh_key"]
        )

        assert bound_ssh_key.id == 2323
        assert bound_ssh_key.name == "My ssh key"
        assert (
            bound_ssh_key.fingerprint
            == "b7:2f:30:a0:2f:6c:58:6c:21:04:58:61:ba:06:3b:2f"
        )
        assert bound_ssh_key.public_key == "ssh-rsa AAAjjk76kgf...Xt"


class TestSSHKeysClient:
    @pytest.fixture()
    def ssh_keys_client(self, client: Client):
        return SSHKeysClient(client)

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        ssh_keys_client: SSHKeysClient,
        ssh_key_response,
    ):
        request_mock.return_value = ssh_key_response

        ssh_key = ssh_keys_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/ssh_keys/1",
        )
        assert ssh_key._client is ssh_keys_client
        assert ssh_key.id == 2323
        assert ssh_key.name == "My ssh key"

    @pytest.mark.parametrize(
        "params",
        [
            {
                "name": "My ssh key",
                "fingerprint": "b7:2f:30:a0:2f:6c:58:6c:21:04:58:61:ba:06:3b:2f",
                "label_selector": "k==v",
                "page": 1,
                "per_page": 10,
            },
            {"name": ""},
            {},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        ssh_keys_client: SSHKeysClient,
        two_ssh_keys_response,
        params,
    ):
        request_mock.return_value = two_ssh_keys_response

        result = ssh_keys_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/ssh_keys",
            params=params,
        )

        ssh_keys = result.ssh_keys
        assert len(ssh_keys) == 2

        ssh_keys1 = ssh_keys[0]
        ssh_keys2 = ssh_keys[1]

        assert ssh_keys1._client is ssh_keys_client
        assert ssh_keys1.id == 2323
        assert ssh_keys1.name == "SSH-Key"

        assert ssh_keys2._client is ssh_keys_client
        assert ssh_keys2.id == 2324
        assert ssh_keys2.name == "SSH-Key"

    @pytest.mark.parametrize(
        "params", [{"name": "My ssh key", "label_selector": "label1"}, {}]
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        ssh_keys_client: SSHKeysClient,
        two_ssh_keys_response,
        params,
    ):
        request_mock.return_value = two_ssh_keys_response

        ssh_keys = ssh_keys_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/ssh_keys",
            params=params,
        )

        assert len(ssh_keys) == 2

        ssh_keys1 = ssh_keys[0]
        ssh_keys2 = ssh_keys[1]

        assert ssh_keys1._client is ssh_keys_client
        assert ssh_keys1.id == 2323
        assert ssh_keys1.name == "SSH-Key"

        assert ssh_keys2._client is ssh_keys_client
        assert ssh_keys2.id == 2324
        assert ssh_keys2.name == "SSH-Key"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        ssh_keys_client: SSHKeysClient,
        one_ssh_keys_response,
    ):
        request_mock.return_value = one_ssh_keys_response

        ssh_keys = ssh_keys_client.get_by_name("SSH-Key")

        params = {"name": "SSH-Key"}

        request_mock.assert_called_with(
            method="GET",
            url="/ssh_keys",
            params=params,
        )

        assert ssh_keys._client is ssh_keys_client
        assert ssh_keys.id == 2323
        assert ssh_keys.name == "SSH-Key"

    def test_get_by_fingerprint(
        self,
        request_mock: mock.MagicMock,
        ssh_keys_client: SSHKeysClient,
        one_ssh_keys_response,
    ):
        request_mock.return_value = one_ssh_keys_response

        ssh_keys = ssh_keys_client.get_by_fingerprint(
            "b7:2f:30:a0:2f:6c:58:6c:21:04:58:61:ba:06:3b:2f"
        )

        params = {"fingerprint": "b7:2f:30:a0:2f:6c:58:6c:21:04:58:61:ba:06:3b:2f"}

        request_mock.assert_called_with(
            method="GET",
            url="/ssh_keys",
            params=params,
        )

        assert ssh_keys._client is ssh_keys_client
        assert ssh_keys.id == 2323
        assert ssh_keys.name == "SSH-Key"

    def test_create(
        self,
        request_mock: mock.MagicMock,
        ssh_keys_client: SSHKeysClient,
        ssh_key_response,
    ):
        request_mock.return_value = ssh_key_response

        ssh_key = ssh_keys_client.create(
            name="My ssh key", public_key="ssh-rsa AAAjjk76kgf...Xt"
        )

        request_mock.assert_called_with(
            method="POST",
            url="/ssh_keys",
            json={"name": "My ssh key", "public_key": "ssh-rsa AAAjjk76kgf...Xt"},
        )

        assert ssh_key.id == 2323
        assert ssh_key.name == "My ssh key"

    @pytest.mark.parametrize(
        "ssh_key", [SSHKey(id=1), BoundSSHKey(mock.MagicMock(), dict(id=1))]
    )
    def test_update(
        self,
        request_mock: mock.MagicMock,
        ssh_keys_client: SSHKeysClient,
        ssh_key,
        response_update_ssh_key,
    ):
        request_mock.return_value = response_update_ssh_key

        ssh_key = ssh_keys_client.update(ssh_key, name="New name")

        request_mock.assert_called_with(
            method="PUT",
            url="/ssh_keys/1",
            json={"name": "New name"},
        )

        assert ssh_key.id == 2323
        assert ssh_key.name == "New name"

    @pytest.mark.parametrize(
        "ssh_key", [SSHKey(id=1), BoundSSHKey(mock.MagicMock(), dict(id=1))]
    )
    def test_delete(
        self,
        request_mock: mock.MagicMock,
        ssh_keys_client: SSHKeysClient,
        ssh_key,
        action_response,
    ):
        request_mock.return_value = action_response

        delete_success = ssh_keys_client.delete(ssh_key)

        request_mock.assert_called_with(
            method="DELETE",
            url="/ssh_keys/1",
        )

        assert delete_success is True



from __future__ import annotations

from unittest import mock

import pytest

from hcloud import Client
from hcloud.firewalls import (
    BoundFirewall,
    Firewall,
    FirewallResource,
    FirewallResourceLabelSelector,
    FirewallRule,
    FirewallsClient,
)
from hcloud.servers import Server

from ..conftest import BoundModelTestCase


class TestBoundFirewall(BoundModelTestCase):
    methods = [
        BoundFirewall.update,
        BoundFirewall.delete,
        BoundFirewall.apply_to_resources,
        BoundFirewall.remove_from_resources,
        BoundFirewall.set_rules,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.firewalls

    @pytest.fixture()
    def bound_model(self, resource_client, firewall_response):
        return BoundFirewall(resource_client, data=firewall_response["firewall"])

    def test_init(self, bound_model: BoundFirewall):
        o = bound_model
        assert o.id == 38
        assert o.name == "Corporate Intranet Protection"
        assert o.labels == {}
        assert isinstance(o.rules, list)
        assert len(o.rules) == 2

        assert isinstance(o.applied_to, list)
        assert len(o.applied_to) == 2
        assert o.applied_to[0].server.id == 42
        assert o.applied_to[0].type == "server"
        assert o.applied_to[1].label_selector.selector == "key==value"
        assert o.applied_to[1].type == "label_selector"

        firewall_in_rule = o.rules[0]
        assert isinstance(firewall_in_rule, FirewallRule)
        assert firewall_in_rule.direction == FirewallRule.DIRECTION_IN
        assert firewall_in_rule.protocol == FirewallRule.PROTOCOL_TCP
        assert firewall_in_rule.port == "80"
        assert isinstance(firewall_in_rule.source_ips, list)
        assert len(firewall_in_rule.source_ips) == 3
        assert firewall_in_rule.source_ips == [
            "28.239.13.1/32",
            "28.239.14.0/24",
            "ff21:1eac:9a3b:ee58:5ca:990c:8bc9:c03b/128",
        ]
        assert isinstance(firewall_in_rule.destination_ips, list)
        assert len(firewall_in_rule.destination_ips) == 0
        assert firewall_in_rule.description == "allow http in"

        firewall_out_rule = o.rules[1]
        assert isinstance(firewall_out_rule, FirewallRule)
        assert firewall_out_rule.direction == FirewallRule.DIRECTION_OUT
        assert firewall_out_rule.protocol == FirewallRule.PROTOCOL_TCP
        assert firewall_out_rule.port == "80"
        assert isinstance(firewall_out_rule.source_ips, list)
        assert len(firewall_out_rule.source_ips) == 0
        assert isinstance(firewall_out_rule.destination_ips, list)
        assert len(firewall_out_rule.destination_ips) == 3
        assert firewall_out_rule.destination_ips == [
            "28.239.13.1/32",
            "28.239.14.0/24",
            "ff21:1eac:9a3b:ee58:5ca:990c:8bc9:c03b/128",
        ]
        assert firewall_out_rule.description == "allow http out"


class TestFirewallsClient:
    @pytest.fixture()
    def firewalls_client(self, client: Client):
        return FirewallsClient(client)

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        firewall_response,
    ):
        request_mock.return_value = firewall_response

        firewall = firewalls_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/firewalls/1",
        )
        assert firewall._client is firewalls_client
        assert firewall.id == 38
        assert firewall.name == "Corporate Intranet Protection"

    @pytest.mark.parametrize(
        "params",
        [
            {
                "name": "Corporate Intranet Protection",
                "sort": "id",
                "label_selector": "k==v",
                "page": 1,
                "per_page": 10,
            },
            {"name": ""},
            {},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        two_firewalls_response,
        params,
    ):
        request_mock.return_value = two_firewalls_response

        result = firewalls_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/firewalls",
            params=params,
        )

        firewalls = result.firewalls
        assert result.meta is not None

        assert len(firewalls) == 2

        firewalls1 = firewalls[0]
        firewalls2 = firewalls[1]

        assert firewalls1._client is firewalls_client
        assert firewalls1.id == 38
        assert firewalls1.name == "Corporate Intranet Protection"

        assert firewalls2._client is firewalls_client
        assert firewalls2.id == 39
        assert firewalls2.name == "Corporate Extranet Protection"

    @pytest.mark.parametrize(
        "params",
        [
            {
                "name": "Corporate Intranet Protection",
                "sort": "id",
                "label_selector": "k==v",
            },
            {},
        ],
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        two_firewalls_response,
        params,
    ):
        request_mock.return_value = two_firewalls_response

        firewalls = firewalls_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/firewalls",
            params=params,
        )

        assert len(firewalls) == 2

        firewalls1 = firewalls[0]
        firewalls2 = firewalls[1]

        assert firewalls1._client is firewalls_client
        assert firewalls1.id == 38
        assert firewalls1.name == "Corporate Intranet Protection"

        assert firewalls2._client is firewalls_client
        assert firewalls2.id == 39
        assert firewalls2.name == "Corporate Extranet Protection"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        one_firewalls_response,
    ):
        request_mock.return_value = one_firewalls_response

        firewall = firewalls_client.get_by_name("Corporate Intranet Protection")

        params = {"name": "Corporate Intranet Protection"}

        request_mock.assert_called_with(
            method="GET",
            url="/firewalls",
            params=params,
        )

        assert firewall._client is firewalls_client
        assert firewall.id == 38
        assert firewall.name == "Corporate Intranet Protection"

    def test_create(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        response_create_firewall,
    ):
        request_mock.return_value = response_create_firewall

        response = firewalls_client.create(
            "Corporate Intranet Protection",
            rules=[
                FirewallRule(
                    direction=FirewallRule.DIRECTION_IN,
                    protocol=FirewallRule.PROTOCOL_ICMP,
                    source_ips=["0.0.0.0/0"],
                )
            ],
            resources=[
                FirewallResource(
                    type=FirewallResource.TYPE_SERVER, server=Server(id=4711)
                ),
                FirewallResource(
                    type=FirewallResource.TYPE_LABEL_SELECTOR,
                    label_selector=FirewallResourceLabelSelector(selector="key==value"),
                ),
            ],
        )

        request_mock.assert_called_with(
            method="POST",
            url="/firewalls",
            json={
                "name": "Corporate Intranet Protection",
                "rules": [
                    {"direction": "in", "protocol": "icmp", "source_ips": ["0.0.0.0/0"]}
                ],
                "apply_to": [
                    {"type": "server", "server": {"id": 4711}},
                    {
                        "type": "label_selector",
                        "label_selector": {"selector": "key==value"},
                    },
                ],
            },
        )

        bound_firewall = response.firewall
        actions = response.actions

        assert bound_firewall._client is firewalls_client
        assert bound_firewall.id == 38
        assert bound_firewall.name == "Corporate Intranet Protection"
        assert len(bound_firewall.applied_to) == 2

        assert len(actions) == 2

    @pytest.mark.parametrize(
        "firewall", [Firewall(id=38), BoundFirewall(mock.MagicMock(), dict(id=38))]
    )
    def test_update(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        firewall,
        response_update_firewall,
    ):
        request_mock.return_value = response_update_firewall

        firewall = firewalls_client.update(
            firewall, name="New Corporate Intranet Protection", labels={}
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/firewalls/38",
            json={"name": "New Corporate Intranet Protection", "labels": {}},
        )

        assert firewall.id == 38
        assert firewall.name == "New Corporate Intranet Protection"

    @pytest.mark.parametrize(
        "firewall", [Firewall(id=1), BoundFirewall(mock.MagicMock(), dict(id=1))]
    )
    def test_set_rules(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        firewall,
        response_set_rules,
    ):
        request_mock.return_value = response_set_rules

        actions = firewalls_client.set_rules(
            firewall,
            [
                FirewallRule(
                    direction=FirewallRule.DIRECTION_IN,
                    protocol=FirewallRule.PROTOCOL_ICMP,
                    source_ips=["0.0.0.0/0", "::/0"],
                    description="Allow ICMP from everywhere",
                ),
                FirewallRule(
                    direction=FirewallRule.DIRECTION_IN,
                    protocol=FirewallRule.PROTOCOL_TCP,
                    port="80",
                    source_ips=["0.0.0.0/0", "::/0"],
                    description="Allow HTTP from everywhere",
                ),
            ],
        )

        request_mock.assert_called_with(
            method="POST",
            url="/firewalls/1/actions/set_rules",
            json={
                "rules": [
                    {
                        "direction": "in",
                        "protocol": "icmp",
                        "source_ips": ["0.0.0.0/0", "::/0"],
                        "description": "Allow ICMP from everywhere",
                    },
                    {
                        "direction": "in",
                        "protocol": "tcp",
                        "port": "80",
                        "source_ips": ["0.0.0.0/0", "::/0"],
                        "description": "Allow HTTP from everywhere",
                    },
                ]
            },
        )

        assert actions[0].id == 13
        assert actions[0].progress == 100

    @pytest.mark.parametrize(
        "firewall", [Firewall(id=1), BoundFirewall(mock.MagicMock(), dict(id=1))]
    )
    def test_delete(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        firewall,
    ):
        delete_success = firewalls_client.delete(firewall)

        request_mock.assert_called_with(
            method="DELETE",
            url="/firewalls/1",
        )

        assert delete_success is True

    @pytest.mark.parametrize(
        "firewall", [Firewall(id=1), BoundFirewall(mock.MagicMock(), dict(id=1))]
    )
    def test_apply_to_resources(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        firewall,
        response_set_rules,
    ):
        request_mock.return_value = response_set_rules

        actions = firewalls_client.apply_to_resources(
            firewall,
            [FirewallResource(type=FirewallResource.TYPE_SERVER, server=Server(id=5))],
        )

        request_mock.assert_called_with(
            method="POST",
            url="/firewalls/1/actions/apply_to_resources",
            json={"apply_to": [{"type": "server", "server": {"id": 5}}]},
        )

        assert actions[0].id == 13
        assert actions[0].progress == 100

    @pytest.mark.parametrize(
        "firewall", [Firewall(id=1), BoundFirewall(mock.MagicMock(), dict(id=1))]
    )
    def test_remove_from_resources(
        self,
        request_mock: mock.MagicMock,
        firewalls_client: FirewallsClient,
        firewall,
        response_set_rules,
    ):
        request_mock.return_value = response_set_rules

        actions = firewalls_client.remove_from_resources(
            firewall,
            [FirewallResource(type=FirewallResource.TYPE_SERVER, server=Server(id=5))],
        )

        request_mock.assert_called_with(
            method="POST",
            url="/firewalls/1/actions/remove_from_resources",
            json={"remove_from": [{"type": "server", "server": {"id": 5}}]},
        )

        assert actions[0].id == 13
        assert actions[0].progress == 100



from __future__ import annotations

import datetime
from datetime import timezone
from unittest import mock

import pytest

from hcloud.networks import Network
from hcloud.servers import (
    BoundServer,
    IPv4Address,
    IPv6Network,
    PrivateNet,
    PublicNetwork,
    PublicNetworkFirewall,
    Server,
    ServerCreatePublicNetwork,
)


@pytest.mark.parametrize(
    "value",
    [
        (Server(id=1),),
        (
            PublicNetwork(
                ipv4=None,
                ipv6=None,
                floating_ips=[],
                primary_ipv4=None,
                primary_ipv6=None,
            ),
        ),
        (PublicNetworkFirewall(firewall=object(), status="pending"),),
        (IPv4Address(ip="127.0.0.1", blocked=False, dns_ptr="example.com"),),
        (IPv6Network("2001:0db8::0/64", blocked=False, dns_ptr="example.com"),),
        (PrivateNet(network=object(), ip="127.0.0.1", alias_ips=[], mac_address=""),),
        (ServerCreatePublicNetwork(),),
    ],
)
def test_eq(value):
    assert value.__eq__(value)


class TestServer:
    def test_created_is_datetime(self):
        server = Server(id=1, created="2016-01-30T23:50+00:00")
        assert server.created == datetime.datetime(
            2016, 1, 30, 23, 50, tzinfo=timezone.utc
        )

    def test_private_net_for(self):
        network1 = Network(id=1)
        network2 = Network(id=2)
        network3 = Network(id=3)

        server = Server(
            id=42,
            private_net=[
                PrivateNet(
                    network=network1, ip="127.0.0.1", alias_ips=[], mac_address=""
                ),
                PrivateNet(
                    network=network2, ip="127.0.0.1", alias_ips=[], mac_address=""
                ),
            ],
        )

        assert server.private_net_for(network1).network.id == 1
        assert server.private_net_for(network3) is None

        server = BoundServer(
            client=mock.MagicMock(),
            data={
                "id": 42,
                "private_net": [
                    {
                        "network": 1,
                        "ip": "127.0.0.1",
                        "alias_ips": [],
                        "mac_address": "",
                    },
                    {
                        "network": 2,
                        "ip": "127.0.0.1",
                        "alias_ips": [],
                        "mac_address": "",
                    },
                ],
            },
        )

        assert server.private_net_for(network1).network.id == 1
        assert server.private_net_for(network3) is None



# pylint: disable=protected-access

from __future__ import annotations

from unittest import mock

import pytest
from dateutil.parser import isoparse

from hcloud import Client
from hcloud.storage_box_types import (
    BoundStorageBoxType,
    StorageBoxTypesClient,
)


def assert_bound_model(
    o: BoundStorageBoxType,
    client: StorageBoxTypesClient,
):
    assert isinstance(o, BoundStorageBoxType)
    assert o._client is client
    assert o.id == 42
    assert o.name == "bx11"


class TestClient:
    @pytest.fixture()
    def resource_client(self, client: Client) -> StorageBoxTypesClient:
        return client.storage_box_types

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxTypesClient,
        storage_box_type1,
    ):
        request_mock.return_value = {"storage_box_type": storage_box_type1}

        result = resource_client.get_by_id(42)

        request_mock.assert_called_with(
            method="GET",
            url="/storage_box_types/42",
        )

        assert_bound_model(result, resource_client)
        assert result.description == "BX11"
        assert result.snapshot_limit == 10
        assert result.automatic_snapshot_limit == 10
        assert result.subaccounts_limit == 100
        assert result.size == 1099511627776
        assert result.prices == [
            {
                "location": "fsn1",
                "price_hourly": {"gross": "0.0051", "net": "0.0051"},
                "price_monthly": {"gross": "3.2000", "net": "3.2000"},
                "setup_fee": {"gross": "0.0000", "net": "0.0000"},
            }
        ]
        assert result.deprecation.announced == isoparse("2023-06-01T00:00:00+00:00")
        assert result.deprecation.unavailable_after == isoparse(
            "2023-09-01T00:00:00+00:00"
        )

    @pytest.mark.parametrize(
        "params",
        [
            {"name": "bx11", "page": 1, "per_page": 10},
            {},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxTypesClient,
        storage_box_type1,
        storage_box_type2,
        params,
    ):
        request_mock.return_value = {
            "storage_box_types": [storage_box_type1, storage_box_type2]
        }

        result = resource_client.get_list(**params)

        request_mock.assert_called_with(
            url="/storage_box_types",
            method="GET",
            params=params,
        )

        assert result.meta is not None
        assert len(result.storage_box_types) == 2

        result1 = result.storage_box_types[0]
        result2 = result.storage_box_types[1]

        assert result1._client is resource_client
        assert result1.id == 42
        assert result1.name == "bx11"

        assert result2._client is resource_client
        assert result2.id == 43
        assert result2.name == "bx21"

    @pytest.mark.parametrize(
        "params",
        [
            {"name": "bx11"},
            {},
        ],
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxTypesClient,
        storage_box_type1,
        storage_box_type2,
        params,
    ):
        request_mock.return_value = {
            "storage_box_types": [storage_box_type1, storage_box_type2]
        }

        result = resource_client.get_all(**params)

        request_mock.assert_called_with(
            url="/storage_box_types",
            method="GET",
            params={**params, "page": 1, "per_page": 50},
        )

        assert len(result) == 2

        result1 = result[0]
        result2 = result[1]

        assert result1._client is resource_client
        assert result1.id == 42
        assert result1.name == "bx11"

        assert result2._client is resource_client
        assert result2.id == 43
        assert result2.name == "bx21"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        resource_client: StorageBoxTypesClient,
        storage_box_type1,
    ):
        request_mock.return_value = {"storage_box_types": [storage_box_type1]}

        result = resource_client.get_by_name("bx11")

        params = {"name": "bx11"}

        request_mock.assert_called_with(
            method="GET",
            url="/storage_box_types",
            params=params,
        )

        assert_bound_model(result, resource_client)



from __future__ import annotations

import pytest

from hcloud.datacenters import Datacenter, DatacenterServerTypes


@pytest.mark.parametrize(
    "value",
    [
        (Datacenter(id=1),),
        (DatacenterServerTypes(available=[], available_for_migration=[], supported=[])),
    ],
)
def test_eq(value):
    assert value.__eq__(value)



from __future__ import annotations

import pytest


@pytest.fixture()
def storage_box1():
    return {
        "id": 42,
        "name": "storage-box1",
        "created": "2025-01-30T23:55:00+00:00",
        "status": "active",
        "system": "FSN1-BX355",
        "server": "u1337.your-storagebox.de",
        "username": "u12345",
        "storage_box_type": {
            "id": 42,
            "name": "bx11",
        },
        "location": {
            "id": 1,
            "name": "fsn1",
        },
        "access_settings": {
            "reachable_externally": False,
            "samba_enabled": False,
            "ssh_enabled": False,
            "webdav_enabled": False,
            "zfs_enabled": False,
        },
        "snapshot_plan": {
            "max_snapshots": 20,
            "minute": 0,
            "hour": 7,
            "day_of_week": 7,
            "day_of_month": None,
        },
        "stats": {
            "size": 2342236717056,
            "size_data": 2102612983808,
            "size_snapshots": 239623733248,
        },
        "labels": {
            "key": "value",
        },
        "protection": {"delete": False},
    }


@pytest.fixture()
def storage_box2():
    return {
        "id": 43,
        "name": "storage-box2",
        "created": "2022-09-30T10:30:09.000Z",
        "status": "active",
        "system": "FSN1-BX355",
        "server": "u1337.your-storagebox.de",
        "username": "u12345",
        "storage_box_type": {
            "id": 1334,
            "name": "bx21",
        },
        "location": {
            "id": 1,
            "name": "fsn1",
        },
        "access_settings": {
            "webdav_enabled": False,
            "zfs_enabled": False,
            "samba_enabled": False,
            "ssh_enabled": True,
            "reachable_externally": True,
        },
        "snapshot_plan": {
            "max_snapshots": 20,
            "minute": 0,
            "hour": 7,
            "day_of_week": 7,
            "day_of_month": None,
        },
        "stats": {
            "size": 2342236717056,
            "size_data": 2102612983808,
            "size_snapshots": 239623733248,
        },
        "labels": {},
        "protection": {"delete": False},
    }


@pytest.fixture()
def storage_box_snapshot1():
    return {
        "id": 34,
        "name": "storage-box-snapshot1",
        "description": "",
        "is_automatic": False,
        "stats": {
            "size": 394957594,
            "size_filesystem": 3949572745,
        },
        "labels": {
            "key": "value",
        },
        "created": "2025-11-10T19:16:57Z",
        "storage_box": 42,
    }


@pytest.fixture()
def storage_box_snapshot2():
    return {
        "id": 35,
        "name": "storage-box-snapshot2",
        "description": "",
        "is_automatic": True,
        "stats": {
            "size": 0,
            "size_filesystem": 0,
        },
        "labels": {},
        "created": "2025-11-10T19:18:57Z",
        "storage_box": 42,
    }


@pytest.fixture()
def storage_box_subaccount1():
    return {
        "id": 45,
        "username": "u42-sub1",
        "server": "u42-sub1.your-storagebox.de",
        "home_directory": "tmp/",
        "description": "Required by foo",
        "access_settings": {
            "samba_enabled": False,
            "ssh_enabled": True,
            "webdav_enabled": False,
            "reachable_externally": True,
            "readonly": False,
        },
        "labels": {
            "key": "value",
        },
        "created": "2025-11-10T19:18:57Z",
        "storage_box": 42,
    }


@pytest.fixture()
def storage_box_subaccount2():
    return {
        "id": 46,
        "username": "u42-sub2",
        "server": "u42-sub2.your-storagebox.de",
        "home_directory": "backup/",
        "description": "",
        "access_settings": {
            "samba_enabled": False,
            "ssh_enabled": True,
            "webdav_enabled": False,
            "reachable_externally": True,
            "readonly": False,
        },
        "labels": {},
        "created": "2025-11-10T19:18:57Z",
        "storage_box": 42,
    }



from __future__ import annotations

from datetime import datetime, timezone
from unittest import mock

import pytest

from hcloud import Client
from hcloud.server_types import BoundServerType, ServerTypesClient


class TestBoundServerType:
    @pytest.fixture()
    def bound_server_type(self, client: Client):
        return BoundServerType(client.server_types, data=dict(id=14))

    def test_init(self, server_type_response):
        o = BoundServerType(
            client=mock.MagicMock(), data=server_type_response["server_type"]
        )

        assert o.id == 1
        assert o.name == "cx11"
        assert o.description == "CX11"
        assert o.category == "Shared vCPU"
        assert o.cores == 1
        assert o.memory == 1
        assert o.disk == 25
        assert o.storage_type == "local"
        assert o.cpu_type == "shared"
        assert o.architecture == "x86"
        assert len(o.locations) == 2
        assert o.locations[0].location.id == 1
        assert o.locations[0].location.name == "nbg1"
        assert o.locations[0].deprecation is None
        assert o.locations[1].location.id == 2
        assert o.locations[1].location.name == "fsn1"
        assert (
            o.locations[1].deprecation.announced.isoformat()
            == "2023-06-01T00:00:00+00:00"
        )
        assert (
            o.locations[1].deprecation.unavailable_after.isoformat()
            == "2023-09-01T00:00:00+00:00"
        )

        with pytest.deprecated_call():
            assert o.deprecated is True
            assert o.deprecation is not None
            assert o.deprecation.announced == datetime(2023, 6, 1, tzinfo=timezone.utc)
            assert o.deprecation.unavailable_after == datetime(
                2023, 9, 1, tzinfo=timezone.utc
            )
            assert o.included_traffic == 21990232555520


class TestServerTypesClient:
    @pytest.fixture()
    def server_types_client(self, client: Client):
        return ServerTypesClient(client)

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        server_types_client: ServerTypesClient,
        server_type_response,
    ):
        request_mock.return_value = server_type_response

        server_type = server_types_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/server_types/1",
        )
        assert server_type._client is server_types_client
        assert server_type.id == 1
        assert server_type.name == "cx11"

    @pytest.mark.parametrize(
        "params", [{"name": "cx11", "page": 1, "per_page": 10}, {"name": ""}, {}]
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        server_types_client: ServerTypesClient,
        two_server_types_response,
        params,
    ):
        request_mock.return_value = two_server_types_response

        result = server_types_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/server_types",
            params=params,
        )

        server_types = result.server_types
        assert result.meta is not None

        assert len(server_types) == 2

        server_types1 = server_types[0]
        server_types2 = server_types[1]

        assert server_types1._client is server_types_client
        assert server_types1.id == 1
        assert server_types1.name == "cx11"

        assert server_types2._client is server_types_client
        assert server_types2.id == 2
        assert server_types2.name == "cx21"

    @pytest.mark.parametrize("params", [{"name": "cx11"}])
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        server_types_client: ServerTypesClient,
        two_server_types_response,
        params,
    ):
        request_mock.return_value = two_server_types_response

        server_types = server_types_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/server_types",
            params=params,
        )

        assert len(server_types) == 2

        server_types1 = server_types[0]
        server_types2 = server_types[1]

        assert server_types1._client is server_types_client
        assert server_types1.id == 1
        assert server_types1.name == "cx11"

        assert server_types2._client is server_types_client
        assert server_types2.id == 2
        assert server_types2.name == "cx21"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        server_types_client: ServerTypesClient,
        one_server_types_response,
    ):
        request_mock.return_value = one_server_types_response

        server_type = server_types_client.get_by_name("cx11")

        params = {"name": "cx11"}

        request_mock.assert_called_with(
            method="GET",
            url="/server_types",
            params=params,
        )

        assert server_type._client is server_types_client
        assert server_type.id == 1
        assert server_type.name == "cx11"



from __future__ import annotations

import datetime
from datetime import timezone
from unittest import mock

import pytest

from hcloud.load_balancers import (
    BoundLoadBalancer,
    IPv4Address,
    IPv6Network,
    LoadBalancer,
    LoadBalancerAlgorithm,
    LoadBalancerHealthCheck,
    LoadBalancerHealthCheckHttp,
    LoadBalancerService,
    LoadBalancerServiceHttp,
    LoadBalancerTarget,
    LoadBalancerTargetHealthStatus,
    LoadBalancerTargetIP,
    LoadBalancerTargetLabelSelector,
    PrivateNet,
    PublicNetwork,
)
from hcloud.networks import Network


@pytest.mark.parametrize(
    "value",
    [
        (LoadBalancer(id=1),),
        (LoadBalancerService,),
        (LoadBalancerServiceHttp(),),
        (LoadBalancerHealthCheck(),),
        (LoadBalancerHealthCheckHttp(),),
        (LoadBalancerTarget(),),
        (LoadBalancerTargetHealthStatus(),),
        (LoadBalancerTargetLabelSelector(),),
        (LoadBalancerTargetIP(),),
        (LoadBalancerAlgorithm(),),
        (
            PublicNetwork(
                ipv4=IPv4Address(ip="127.0.0.1", dns_ptr="example.com"),
                ipv6=IPv6Network("2001:0db8::0/64", dns_ptr="example.com"),
                enabled=True,
            ),
        ),
        (IPv4Address(ip="127.0.0.1", dns_ptr="example.com"),),
        (IPv6Network("2001:0db8::0/64", dns_ptr="example.com"),),
        (PrivateNet(network=object(), ip="127.0.0.1"),),
    ],
)
def test_eq(value):
    assert value.__eq__(value)


class TestLoadBalancers:
    def test_created_is_datetime(self):
        lb = LoadBalancer(id=1, created="2016-01-30T23:50+00:00")
        assert lb.created == datetime.datetime(2016, 1, 30, 23, 50, tzinfo=timezone.utc)

    def test_private_net_for(self):
        network1 = Network(id=1)
        network2 = Network(id=2)
        network3 = Network(id=3)

        load_balancer = LoadBalancer(
            id=42,
            private_net=[
                PrivateNet(network=network1, ip="127.0.0.1"),
                PrivateNet(network=network2, ip="127.0.0.1"),
            ],
        )

        assert load_balancer.private_net_for(network1).network.id == 1
        assert load_balancer.private_net_for(network3) is None

        load_balancer = BoundLoadBalancer(
            client=mock.MagicMock(),
            data={
                "id": 42,
                "private_net": [
                    {"network": 1, "ip": "127.0.0.1"},
                    {"network": 2, "ip": "127.0.0.1"},
                ],
            },
        )

        assert load_balancer.private_net_for(network1).network.id == 1
        assert load_balancer.private_net_for(network3) is None



from __future__ import annotations

from unittest import mock

import pytest

from hcloud import Client
from hcloud.load_balancer_types import LoadBalancerType
from hcloud.load_balancers import (
    BoundLoadBalancer,
    LoadBalancer,
    LoadBalancerAlgorithm,
    LoadBalancerHealthCheck,
    LoadBalancersClient,
    LoadBalancerService,
    LoadBalancerTarget,
    LoadBalancerTargetIP,
    LoadBalancerTargetLabelSelector,
)
from hcloud.locations import Location
from hcloud.networks import Network
from hcloud.servers import BoundServer, Server

from ..conftest import BoundModelTestCase


class TestBoundLoadBalancer(BoundModelTestCase):
    methods = [
        BoundLoadBalancer.update,
        BoundLoadBalancer.delete,
        BoundLoadBalancer.change_algorithm,
        BoundLoadBalancer.change_dns_ptr,
        BoundLoadBalancer.change_protection,
        BoundLoadBalancer.change_type,
        BoundLoadBalancer.add_service,
        BoundLoadBalancer.update_service,
        BoundLoadBalancer.delete_service,
        BoundLoadBalancer.add_target,
        BoundLoadBalancer.remove_target,
        BoundLoadBalancer.attach_to_network,
        BoundLoadBalancer.detach_from_network,
        BoundLoadBalancer.disable_public_interface,
        BoundLoadBalancer.enable_public_interface,
        BoundLoadBalancer.get_metrics,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.load_balancers

    @pytest.fixture()
    def bound_model(self, resource_client: LoadBalancersClient):
        return BoundLoadBalancer(resource_client, data=dict(id=1))

    def test_init(self, response_load_balancer):
        bound_load_balancer = BoundLoadBalancer(
            client=mock.MagicMock(), data=response_load_balancer["load_balancer"]
        )

        assert bound_load_balancer.id == 4711
        assert bound_load_balancer.name == "Web Frontend"

    def test_init_label_selector_nested_targets(self, response_load_balancer):
        bound_load_balancer = BoundLoadBalancer(
            client=mock.MagicMock(), data=response_load_balancer["load_balancer"]
        )

        label_selector_target = bound_load_balancer.targets[1]
        assert label_selector_target.type == "label_selector"
        assert label_selector_target.label_selector.selector == "env=prod"
        assert label_selector_target.use_private_ip is True
        assert label_selector_target.targets is not None
        assert len(label_selector_target.targets) == 1

        nested = label_selector_target.targets[0]
        assert nested.type == "server"
        assert isinstance(nested.server, BoundServer)
        assert nested.server.id == 105054278
        assert nested.use_private_ip is True
        assert nested.health_status is not None
        assert len(nested.health_status) == 2
        assert nested.health_status[0].listen_port == 443
        assert nested.health_status[0].status == "healthy"
        assert nested.health_status[1].listen_port == 3000
        assert nested.health_status[1].status == "healthy"


class TestLoadBalancerslient:
    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.load_balancers

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        response_load_balancer,
    ):
        request_mock.return_value = response_load_balancer

        bound_load_balancer = resource_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/load_balancers/1",
        )
        assert bound_load_balancer._client is resource_client
        assert bound_load_balancer.id == 4711
        assert bound_load_balancer.name == "Web Frontend"
        assert bound_load_balancer.outgoing_traffic == 123456
        assert bound_load_balancer.ingoing_traffic == 123456
        assert bound_load_balancer.included_traffic == 654321

    @pytest.mark.parametrize(
        "params",
        [
            {
                "name": "load_balancer1",
                "label_selector": "label1",
                "page": 1,
                "per_page": 10,
            },
            {"name": ""},
            {},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        response_simple_load_balancers,
        params,
    ):
        request_mock.return_value = response_simple_load_balancers

        result = resource_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/load_balancers",
            params=params,
        )

        bound_load_balancers = result.load_balancers
        assert result.meta is not None

        assert len(bound_load_balancers) == 2

        bound_load_balancer1 = bound_load_balancers[0]
        bound_load_balancer2 = bound_load_balancers[1]

        assert bound_load_balancer1._client is resource_client
        assert bound_load_balancer1.id == 4711
        assert bound_load_balancer1.name == "Web Frontend"

        assert bound_load_balancer2._client is resource_client
        assert bound_load_balancer2.id == 4712
        assert bound_load_balancer2.name == "Web Frontend2"

    @pytest.mark.parametrize(
        "params", [{"name": "loadbalancer1", "label_selector": "label1"}, {}]
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        response_simple_load_balancers,
        params,
    ):
        request_mock.return_value = response_simple_load_balancers

        bound_load_balancers = resource_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/load_balancers",
            params=params,
        )

        assert len(bound_load_balancers) == 2

        bound_load_balancer1 = bound_load_balancers[0]
        bound_load_balancer2 = bound_load_balancers[1]

        assert bound_load_balancer1._client is resource_client
        assert bound_load_balancer1.id == 4711
        assert bound_load_balancer1.name == "Web Frontend"

        assert bound_load_balancer2._client is resource_client
        assert bound_load_balancer2.id == 4712
        assert bound_load_balancer2.name == "Web Frontend2"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        response_simple_load_balancers,
    ):
        request_mock.return_value = response_simple_load_balancers

        bound_load_balancer = resource_client.get_by_name("Web Frontend")

        params = {"name": "Web Frontend"}

        request_mock.assert_called_with(
            method="GET",
            url="/load_balancers",
            params=params,
        )

        assert bound_load_balancer._client is resource_client
        assert bound_load_balancer.id == 4711
        assert bound_load_balancer.name == "Web Frontend"

    def test_create(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        response_create_load_balancer,
    ):
        request_mock.return_value = response_create_load_balancer

        response = resource_client.create(
            "my-balancer",
            load_balancer_type=LoadBalancerType(name="lb11"),
            location=Location(id=1),
        )

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers",
            json={"name": "my-balancer", "load_balancer_type": "lb11", "location": 1},
        )

        bound_load_balancer = response.load_balancer

        assert bound_load_balancer._client is resource_client
        assert bound_load_balancer.id == 1
        assert bound_load_balancer.name == "my-balancer"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_change_type_with_load_balancer_type_name(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.change_type(
            load_balancer, LoadBalancerType(name="lb11")
        )

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/change_type",
            json={"load_balancer_type": "lb11"},
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_change_type_with_load_balancer_type_id(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.change_type(load_balancer, LoadBalancerType(id=1))

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/change_type",
            json={"load_balancer_type": 1},
        )

        assert action.id == 1
        assert action.progress == 0

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_update(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_update_load_balancer,
    ):
        request_mock.return_value = response_update_load_balancer

        load_balancer = resource_client.update(
            load_balancer, name="new-name", labels={}
        )

        request_mock.assert_called_with(
            method="PUT",
            url="/load_balancers/1",
            json={"name": "new-name", "labels": {}},
        )

        assert load_balancer.id == 4711
        assert load_balancer.name == "new-name"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_delete(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        action_response,
    ):
        request_mock.return_value = action_response

        delete_success = resource_client.delete(load_balancer)

        request_mock.assert_called_with(
            method="DELETE",
            url="/load_balancers/1",
        )

        assert delete_success is True

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_get_metrics(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_get_metrics,
    ):
        request_mock.return_value = response_get_metrics

        response = resource_client.get_metrics(
            load_balancer,
            type=["requests_per_second"],
            start="2023-12-14T16:55:32+01:00",
            end="2023-12-14T16:55:32+01:00",
        )

        request_mock.assert_called_with(
            method="GET",
            url="/load_balancers/1/metrics",
            params={
                "type": "requests_per_second",
                "start": "2023-12-14T16:55:32+01:00",
                "end": "2023-12-14T16:55:32+01:00",
            },
        )
        assert "requests_per_second" in response.metrics.time_series
        assert len(response.metrics.time_series["requests_per_second"]["values"]) == 3

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_add_service(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_add_service,
    ):
        request_mock.return_value = response_add_service

        service = LoadBalancerService(listen_port=80, protocol="http")
        action = resource_client.add_service(load_balancer, service)

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/add_service",
            json={"protocol": "http", "listen_port": 80},
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "add_service"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_delete_service(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_delete_service,
    ):
        request_mock.return_value = response_delete_service

        service = LoadBalancerService(listen_port=12)
        action = resource_client.delete_service(load_balancer, service)

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/delete_service",
            json={"listen_port": 12},
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "delete_service"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    @pytest.mark.parametrize(
        "target,params",
        [
            (
                LoadBalancerTarget(
                    type="server", server=Server(id=1), use_private_ip=True
                ),
                {"server": {"id": 1}, "use_private_ip": True},
            ),
            (
                LoadBalancerTarget(type="ip", ip=LoadBalancerTargetIP(ip="127.0.0.1")),
                {"ip": {"ip": "127.0.0.1"}},
            ),
            (
                LoadBalancerTarget(
                    type="label_selector",
                    label_selector=LoadBalancerTargetLabelSelector(selector="abc=def"),
                ),
                {"label_selector": {"selector": "abc=def"}},
            ),
        ],
    )
    def test_add_target(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_add_target,
        target,
        params,
    ):
        request_mock.return_value = response_add_target

        action = resource_client.add_target(load_balancer, target)
        params.update({"type": target.type})

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/add_target",
            json=params,
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "add_target"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    @pytest.mark.parametrize(
        "target,params",
        [
            (
                LoadBalancerTarget(
                    type="server", server=Server(id=1), use_private_ip=True
                ),
                {"server": {"id": 1}},
            ),
            (
                LoadBalancerTarget(type="ip", ip=LoadBalancerTargetIP(ip="127.0.0.1")),
                {"ip": {"ip": "127.0.0.1"}},
            ),
            (
                LoadBalancerTarget(
                    type="label_selector",
                    label_selector=LoadBalancerTargetLabelSelector(selector="abc=def"),
                ),
                {"label_selector": {"selector": "abc=def"}},
            ),
        ],
    )
    def test_remove_target(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_remove_target,
        target,
        params,
    ):
        request_mock.return_value = response_remove_target

        action = resource_client.remove_target(load_balancer, target)
        params.update({"type": target.type})

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/remove_target",
            json=params,
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "remove_target"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_update_service(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_update_service,
    ):
        request_mock.return_value = response_update_service

        new_health_check = LoadBalancerHealthCheck(
            protocol="http", port=13, interval=1, timeout=1, retries=1
        )
        service = LoadBalancerService(listen_port=12, health_check=new_health_check)

        action = resource_client.update_service(load_balancer, service)

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/update_service",
            json={
                "listen_port": 12,
                "health_check": {
                    "protocol": "http",
                    "port": 13,
                    "interval": 1,
                    "timeout": 1,
                    "retries": 1,
                },
            },
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "update_service"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_change_algorithm(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_change_algorithm,
    ):
        request_mock.return_value = response_change_algorithm

        algorithm = LoadBalancerAlgorithm(type="round_robin")
        action = resource_client.change_algorithm(load_balancer, algorithm)

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/change_algorithm",
            json={"type": "round_robin"},
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "change_algorithm"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_change_dns_ptr(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_change_reverse_dns_entry,
    ):
        request_mock.return_value = response_change_reverse_dns_entry

        action = resource_client.change_dns_ptr(
            load_balancer, ip="1.2.3.4", dns_ptr="lb1.example.com"
        )

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/change_dns_ptr",
            json={"dns_ptr": "lb1.example.com", "ip": "1.2.3.4"},
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "change_dns_ptr"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_change_protection(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_change_protection,
    ):
        request_mock.return_value = response_change_protection

        action = resource_client.change_protection(load_balancer, delete=True)

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/change_protection",
            json={"delete": True},
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "change_protection"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_enable_public_interface(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_enable_public_interface,
    ):
        request_mock.return_value = response_enable_public_interface

        action = resource_client.enable_public_interface(load_balancer)

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/enable_public_interface",
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "enable_public_interface"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_disable_public_interface(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_disable_public_interface,
    ):
        request_mock.return_value = response_disable_public_interface

        action = resource_client.disable_public_interface(load_balancer)

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/disable_public_interface",
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "disable_public_interface"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_attach_to_network(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_attach_load_balancer_to_network,
    ):
        request_mock.return_value = response_attach_load_balancer_to_network

        action = resource_client.attach_to_network(load_balancer, Network(id=1))

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/attach_to_network",
            json={"network": 1},
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "attach_to_network"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_detach_from_network(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        response_detach_from_network,
    ):
        request_mock.return_value = response_detach_from_network

        action = resource_client.detach_from_network(load_balancer, Network(id=1))

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/detach_from_network",
            json={"network": 1},
        )

        assert action.id == 13
        assert action.progress == 100
        assert action.command == "detach_from_network"

    @pytest.mark.parametrize(
        "load_balancer",
        [LoadBalancer(id=1), BoundLoadBalancer(mock.MagicMock(), dict(id=1))],
    )
    def test_change_type(
        self,
        request_mock: mock.MagicMock,
        resource_client: LoadBalancersClient,
        load_balancer,
        action_response,
    ):
        request_mock.return_value = action_response

        action = resource_client.change_type(
            load_balancer, LoadBalancerType(name="lb21")
        )

        request_mock.assert_called_with(
            method="POST",
            url="/load_balancers/1/actions/change_type",
            json={"load_balancer_type": "lb21"},
        )

        assert action.id == 1
        assert action.progress == 0



from __future__ import annotations

from unittest import mock

import pytest

from hcloud import Client
from hcloud.certificates import (
    BoundCertificate,
    Certificate,
    CertificatesClient,
    ManagedCertificateStatus,
)

from ..conftest import BoundModelTestCase


class TestBoundCertificate(BoundModelTestCase):
    methods = [
        BoundCertificate.update,
        BoundCertificate.delete,
        BoundCertificate.retry_issuance,
    ]

    @pytest.fixture()
    def resource_client(self, client: Client):
        return client.certificates

    @pytest.fixture()
    def bound_model(self, resource_client, certificate_response):
        return BoundCertificate(
            resource_client, data=certificate_response["certificate"]
        )

    def test_init(self, bound_model: BoundCertificate):
        o = bound_model
        assert o.id == 2323
        assert o.name == "My Certificate"
        assert o.type == "managed"
        assert o.fingerprint == "03:c7:55:9b:2a:d1:04:17:09:f6:d0:7f:18:34:63:d4:3e:5f"
        assert o.certificate == "-----BEGIN CERTIFICATE-----\n..."
        assert len(o.domain_names) == 3
        assert o.domain_names[0] == "example.com"
        assert o.domain_names[1] == "webmail.example.com"
        assert o.domain_names[2] == "www.example.com"
        assert isinstance(o.status, ManagedCertificateStatus)
        assert o.status.issuance == "failed"
        assert o.status.renewal == "scheduled"
        assert o.status.error.code == "error_code"
        assert o.status.error.message == "error message"


class TestCertificatesClient:
    @pytest.fixture()
    def certificates_client(self, client: Client):
        return CertificatesClient(client)

    def test_get_by_id(
        self,
        request_mock: mock.MagicMock,
        certificates_client: CertificatesClient,
        certificate_response,
    ):
        request_mock.return_value = certificate_response

        certificate = certificates_client.get_by_id(1)

        request_mock.assert_called_with(
            method="GET",
            url="/certificates/1",
        )
        assert certificate._client is certificates_client
        assert certificate.id == 2323
        assert certificate.name == "My Certificate"

    @pytest.mark.parametrize(
        "params",
        [
            {
                "name": "My Certificate",
                "label_selector": "k==v",
                "page": 1,
                "per_page": 10,
            },
            {"name": ""},
            {},
        ],
    )
    def test_get_list(
        self,
        request_mock: mock.MagicMock,
        certificates_client: CertificatesClient,
        two_certificates_response,
        params,
    ):
        request_mock.return_value = two_certificates_response

        result = certificates_client.get_list(**params)

        request_mock.assert_called_with(
            method="GET",
            url="/certificates",
            params=params,
        )

        certificates = result.certificates
        assert len(certificates) == 2

        certificates1 = certificates[0]
        certificates2 = certificates[1]

        assert certificates1._client is certificates_client
        assert certificates1.id == 2323
        assert certificates1.name == "My Certificate"

        assert certificates2._client is certificates_client
        assert certificates2.id == 2324
        assert certificates2.name == "My website cert"

    @pytest.mark.parametrize(
        "params", [{"name": "My Certificate", "label_selector": "label1"}, {}]
    )
    def test_get_all(
        self,
        request_mock: mock.MagicMock,
        certificates_client: CertificatesClient,
        two_certificates_response,
        params,
    ):
        request_mock.return_value = two_certificates_response

        certificates = certificates_client.get_all(**params)

        params.update({"page": 1, "per_page": 50})

        request_mock.assert_called_with(
            method="GET",
            url="/certificates",
            params=params,
        )

        assert len(certificates) == 2

        certificates1 = certificates[0]
        certificates2 = certificates[1]

        assert certificates1._client is certificates_client
        assert certificates1.id == 2323
        assert certificates1.name == "My Certificate"

        assert certificates2._client is certificates_client
        assert certificates2.id == 2324
        assert certificates2.name == "My website cert"

    def test_get_by_name(
        self,
        request_mock: mock.MagicMock,
        certificates_client: CertificatesClient,
        one_certificates_response,
    ):
        request_mock.return_value = one_certificates_response

        certificates = certificates_client.get_by_name("My Certificate")

        params = {"name": "My Certificate"}

        request_mock.assert_called_with(
            method="GET",
            url="/certificates",
            params=params,
        )

        assert certificates._client is certificates_client
        assert certificates.id == 2323
        assert certificates.name == "My Certificate"

    def test_create(
        self,
        request_mock: mock.MagicMock,
        certificates_client: CertificatesClient,
        certificate_response,
    ):
        request_mock.return_value = certificate_response

        certificate = certificates_client.create(
            name="My Certificate",
            certificate="-----BEGIN CERTIFICATE-----\n...",
            private_key="-----BEGIN PRIVATE KEY-----\n...",
        )

        request_mock.assert_called_with(
            method="POST",
            url="/certificates",
            json={
                "name": "My Certificate",
                "certificate": "-----BEGIN CERTIFICATE-----\n...",
                "private_key": "-----BEGIN PRIVATE KEY-----\n...",
                "type": "uploaded",
            },
        )

        assert certificate.id == 2323
        assert certificate.name == "My Certificate"

    def test_create_managed(
        self,
        request_mock: mock.MagicMock,
        certificates_client: CertificatesClient,
        create_managed_certificate_response,
    ):
        request_mock.return_value = create_managed_certificate_response

        create_managed_certificate_rsp = certificates_client.create_managed(
            name="My Certificate", domain_names=["example.com", "*.example.org"]
        )

        request_mock.assert_called_with(
            method="POST",
            url="/certificates",
            json={
                "name": "My Certificate",
                "domain_names": ["example.com", "*.example.org"],
                "type": "managed",
            },
        )

        assert create_managed_certificate_rsp.certificate.id == 2323
        assert create_managed_certificate_rsp.certificate.name == "My Certificate"
        assert create_managed_certificate_rsp.action.id == 14
        assert create_managed_certificate_rsp.action.command == "issue_certificate"

    @pytest.mark.parametrize(
        "certificate",
        [Certificate(id=1), BoundCertificate(mock.MagicMock(), dict(id=1))],
    )
    def test_update(
        self,
        request_mock: mock.MagicMock,
        certificates_client: CertificatesClient,
        certificate,
        response_update_certificate,
    ):
        request_mock.return_value = response_update_certificate

        certificate = certificates_client.update(certificate, name="New name")

        request_mock.assert_called_with(
            method="PUT",
            url="/certificates/1",
            json={"name": "New name"},
        )

        assert certificate.id == 2323
        assert certificate.name == "New name"

    @pytest.mark.parametrize(
        "certificate",
        [Certificate(id=1), BoundCertificate(mock.MagicMock(), dict(id=1))],
    )
    def test_delete(
        self,
        request_mock: mock.MagicMock,
        certificates_client: CertificatesClient,
        certificate,
        action_response,
    ):
        request_mock.return_value = action_response

        delete_success = certificates_client.delete(certificate)

        request_mock.assert_called_with(
            method="DELETE",
            url="/certificates/1",
        )

        assert delete_success is True

    @pytest.mark.parametrize(
        "certificate",
        [Certificate(id=1), BoundCertificate(mock.MagicMock(), dict(id=1))],
    )
    def test_retry_issuance(
        self,
        request_mock: mock.MagicMock,
        certificates_client: CertificatesClient,
        certificate,
        response_retry_issuance_action,
    ):
        request_mock.return_value = response_retry_issuance_action

        action = certificates_client.retry_issuance(certificate)

        request_mock.assert_called_with(
            method="POST",
            url="/certificates/1/actions/retry",
        )

        assert action.id == 14
        assert action.command == "issue_certificate"
